/*
 * @brief This header file is part of Ding RTC engine sdk for iOS/Mac system
 *
 * @copyright Copyright (c) 2021, Alibaba Inc.
 * All rights reserved
 */

#import <TargetConditionals.h>

#if TARGET_OS_IPHONE
#import <UIKit/UIKit.h>
typedef UIColor DingRtcOSColor;
typedef UIImage DingRtcOSImage;
typedef UIView DingRtcOSView;
typedef UITextView DingRtcOSTextView;
#else
#import <AppKit/AppKit.h>
typedef NSColor DingRtcOSColor;
typedef NSImage DingRtcOSImage;
typedef NSView DingRtcOSView;
typedef NSTextView DingRtcOSTextView;
#endif
#import <CoreVideo/CoreVideo.h>

/**
 * @defgroup OC_DingRtcEngine 全部接口
 * DingRtc 接口
 */

/** @defgroup OC_DingRtcEngineBase 基础接口
 *  @ingroup OC_DingRtcEngine
 *  DingRtc 基础接口
 */

/** @defgroup OC_DingRtcEngineChannel 频道相关接口
 *  @ingroup OC_DingRtcEngine
 *  DingRtc 频道相关接口
 */

/** @defgroup OC_DingRtcEnginePub 发布相关接口
 *  @ingroup OC_DingRtcEngine
 *  DingRtc 发布相关接口
 */

/** @defgroup OC_DingRtcEngineSub 订阅相关接口
 *  @ingroup OC_DingRtcEngine
 *  DingRtc 订阅相关接口
 */

/** @defgroup OC_DingRtcEngineVideo 视频相关接口
 *  @ingroup OC_DingRtcEngine
 *  DingRtc 视频相关接口
 */

/** @defgroup OC_DingRtcEngineScreenShare 共享视频相关接口
 *  @ingroup OC_DingRtcEngine
 *  DingRtc 共享视频相关接口
 */

/** @defgroup OC_DingRtcEngineAudio 音频相关接口
 *  @ingroup OC_DingRtcEngine
 *  DingRtc 音频相关接口
 */

/** @defgroup OC_DingRtcEngineMediaEngine 媒体引擎相关接口
 *  @ingroup OC_DingRtcEngine
 *  DingRtc 媒体引擎相关接口
 */

/** @defgroup OC_DingRtcEngineLiveBypass 直播旁路相关接口
 *  @ingroup OC_DingRtcEngine
 *  DingRtc 直播旁路相关接口
 */

/** @defgroup OC_DingRtcEngineChannelRelay 跨频道连麦相关接口
 *  @ingroup OC_DingRtcEngine
 *  DingRtc 跨频道连麦相关接口
 */

/** @defgroup OC_DingRtcEnginePreview 预览相关接口
 *  @ingroup OC_DingRtcEngine
 *  DingRtc 预览相关接口
 */

/** @defgroup OC_DingRtcEngineUserInfo 远端用户查询接口
 *  @ingroup OC_DingRtcEngine
 *  DingRtc 远端用户查询接口
 */

/** @defgroup OC_DingRtcEngineOthers 其他接口
 *  @ingroup OC_DingRtcEngine
 *  DingRtc 其他接口
 */

/**
* @defgroup OC_DingRtcDef 关键类型定义
* DingRtc 关键类型定义
*/

/**
* @defgroup OC_DingRtcCallback 回调接口
* DingRtc 关键类型定义
*/

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 视频流类型。
 */
typedef NS_ENUM(NSUInteger, DingRtcVideoTrack) {
    /** 无摄像头和屏幕共享。 */
    DingRtcVideoTrackNo     = 0,
    /** 摄像头。 */
    DingRtcVideoTrackCamera = 1,
    /** 屏幕共享。 */
    DingRtcVideoTrackScreen = 2,
    /** 摄像头+屏幕共享。 */
    DingRtcVideoTrackBoth   = 3,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 音频流类型。
 */
typedef NS_ENUM(NSUInteger, DingRtcAudioTrack) {
    /** 无麦克风。 */
    DingRtcAudioTrackNo   = 0,
    /** 麦克风。 */
    DingRtcAudioTrackMic  = 1,
};


/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 网络质量。
 
 - DingRtcNetworkQualityGood：网络质量好。
 - DingRtcNetworkQualityPoor：网络质量差。
 - DingRtcNetworkQualityDisconnect：网络中断。
 - DingRtcNetworkQualityUnknow：未知。
 */
typedef NS_ENUM(NSUInteger, DingRtcNetworkQuality) {
    /** 网络质量好。 */
    DingRtcNetworkQualityGood       = 0,
    /** 网络质量差。 */
    DingRtcNetworkQualityPoor       = 1,
    /** 网络中断。 */
    DingRtcNetworkQualityDisconnect = 2,
    /** 未知。 */
    DingRtcNetworkQualityUnknown     = 3,
};


/**
 * @brief 音频质量

- DingRtcAudioQulityUnknow 质量未知
- DingRtcAudioQulityExcellent 质量极好
- DingRtcAudioQulityGood 用户主观感觉和极好差不多 ，但码率可能略低于极好
- DingRtcAudioQulityPoor 用户主观感受有瑕疵，但不影响沟通
- DingRtcAudioQulityBad 勉强能沟通但不顺畅
- DingRtcAudioQulityVeryBad 网络质量非常差，基本不能沟通
- DingRtcAudioQulityNetworkDisconnected 网络连接已断开，完全无法沟通
*/
typedef NS_ENUM(NSInteger, DingRtcTranportAudioQuality) {
    /** 质量未知。 */
    DingRtcAudioQulityUnknow = 0,
    /** 质量极好。 */
    DingRtcAudioQulityExcellent = 1,
    /** 用户主观感觉和极好差不多 ，但码率可能略低于极好。 */
    DingRtcAudioQulityGood = 2,
    /** 用户主观感受有瑕疵，但不影响沟通。 */
    DingRtcAudioQulityPoor = 3,
    /** 勉强能沟通但不顺畅。 */
    DingRtcAudioQulityBad = 4,
    /** 网络质量非常差，基本不能沟通。 */
    DingRtcAudioQulityVeryBad = 5,
    /** 网络连接已断开，完全无法沟通。 */
    DingRtcAudioQulityNetworkDisconnected = 6,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 音频编码模式。
 */
typedef NS_ENUM(NSInteger, DingRtcAudioProfile) {
    /** 标准音质模式，默认16000Hz采样率，单声道，最大编码码率24kbps。 */
    DingRtcEngineBasicQualityMode      = 0x0000,
    /** 高音质模式，默认48000Hz采样率，单声道，最大编码码率48kbps。 */
    DingRtcEngineHighQualityMode       = 0x0001,
    /** 超高音质模式，默认48000Hz采样率，单声道，最大编码码率96kbps。 */
    DingRtcEngineSuperHighQualityMode = 0x0002,
    // /** 立体声高音质模式，默认48000Hz采样率，双声道，最大编码码率80kbps。 */
    // DingRtcEngineStereoHighQualityMode = 0x0003,
    // /** 立体声超高音质模式，默认48000Hz采样率，双声道，最大编码码率128kbps。 */
    // DingRtcEngineStereoSuperHighQualityMode = 0x0004,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 音频场景模式。
 */
typedef NS_ENUM(NSInteger, DingRtcAudioScenario) {
    /** 默认场景，一般的音视频通信场景推荐使用。 */
    DingRtcSceneDefaultMode   = 0x0000,
    // /** 音乐场景，高音质场景，适用于音乐为主的场景。 */
    // DingRtcSceneMusicMode     = 0x0001,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 视图显示模式。
 */
typedef NS_ENUM(NSUInteger, DingRtcRenderMode) {
    /** 自动模式。 */
    DingRtcRenderModeAuto    = 0,
    /** 延伸模式。 */
    DingRtcRenderModeStretch = 1,
    /** 填充模式。 */
    DingRtcRenderModeFill    = 2,
    /** 裁剪模式。 */
    DingRtcRenderModeCrop    = 3,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief Log级别。
 */
typedef NS_ENUM(NSUInteger, DingRtcLogLevel) {
    /** 只输出>=DingRtcLogLevelInfo 级别的日志。 */
    DingRtcLogLevelInfo = 3,
    /** 只输出>=DingRtcLogLevelWarn 级别的日志。 */
    DingRtcLogLevelWarn = 4,
    /** 只输出>=DingRtcLogLevelError 级别的日志。 */
    DingRtcLogLevelError = 5,
    /** 不输出日志。 */
    DingRtcLogLevelNone = 6,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 视频编码帧率。
 */
typedef NS_ENUM(NSInteger, DingRtcVideoEncoderFrameRate) {
    DingRtcVideoEncoderFrameRate_FPS_5 = 5,
    DingRtcVideoEncoderFrameRate_FPS_10 = 10,
    DingRtcVideoEncoderFrameRate_FPS_15 = 15,
    DingRtcVideoEncoderFrameRate_FPS_20 = 20,
    DingRtcVideoEncoderFrameRate_FPS_30 = 30,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 编码视频镜像模式。

 - DingRtcVideoEncoderMirrorModeDisabled：编码视频镜像
 - DingRtcVideoEncoderMirrorModeEnabled：编码视频不镜像
 */
typedef NS_ENUM(NSInteger, DingRtcVideoEncoderMirrorMode) {
    /** 编码视频镜像。 */
    DingRtcVideoEncoderMirrorModeDisabled = 0,
    /** 编码视频不镜像。 */
    DingRtcVideoEncoderMirrorModeEnabled = 1,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 视频旋转角度。
 */
typedef NS_ENUM(NSInteger, DingRtcRotationMode) {
    /** 旋转0度。 */
  DingRtcRotationMode_0    = 0,
    /** 旋转90度。 */
  DingRtcRotationMode_90   = 90,
    /** 旋转180度。 */
  DingRtcRotationMode_180  = 180,
    /** 旋转270度。 */
  DingRtcRotationMode_270  = 270,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 编码视频旋转模式。

 - DingRtcVideoEncoderOrientationModeAdaptive：自适应，和采集视频保持一致
 - DingRtcVideoEncoderOrientationModeFixedLandscape：固定横屏
 - DingRtcVideoEncoderOrientationModeFixedPortrait：固定竖屏
 */
typedef NS_ENUM(NSInteger, DingRtcVideoEncoderOrientationMode) {
    /** 自适应，和采集视频保持一致。 */
    DingRtcVideoEncoderOrientationModeAdaptive = 0,
    /** 固定横屏。 */
    DingRtcVideoEncoderOrientationModeFixedLandscape = 1,
    /** 固定竖屏。 */
    DingRtcVideoEncoderOrientationModeFixedPortrait = 2,
};

#if TARGET_OS_IPHONE
/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @remark ios
 * @brief 设备方向 (仅适用于 iOS 系统)。
 - DingRtcOrientationModePortrait：竖屏模式
 - DingRtcOrientationModeLandscapeLeft：左横屏模式
 - DingRtcOrientationModePortraitUpsideDown：倒立模式
 - DingRtcOrientationModeLandscapeRight：右横屏模式
 - DingRtcOrientationModeAuto：由SDK自己决定
 */
typedef NS_ENUM(NSInteger, DingRtcOrientationMode) {
    /** 竖屏模式。 */
    DingRtcOrientationModePortrait = 0,
    /** 左横屏模式。 */
    DingRtcOrientationModeLandscapeLeft = 90,
    /** 倒立模式。 */
    DingRtcOrientationModePortraitUpsideDown = 180,
    /** 右横屏模式。 */
    DingRtcOrientationModeLandscapeRight = 270,
    /** 由SDK自己决定。 */
    DingRtcOrientationModeAuto = 4,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @remark ios
 * @brief SDK对Audio Session的控制权限 (仅适用于 iOS 系统)。
 
 - DingRtcAudioSessionOperationRestrictionNone：没有限制，SDK完全控制AVAudioSession
 - DingRtcAudioSessionOperationRestrictionSetCategory：SDK不能修改AVAudioSession的Category
 - DingRtcAudioSessionOperationRestrictionConfigureSession：SDK不能修改AVAudioSession的配置，包括Category，Mode，CategoryOptions
 - DingRtcAudioSessionOperationRestrictionDeactivateSession：SDK不能关闭AVAudioSession的活动状态，离开频道时，AVAudioSession依然处于活动状态
 */
typedef NS_ENUM(NSInteger, DingRtcAudioSessionOperationRestriction) {
    /** 没有限制，SDK完全控制AVAudioSession。 */
    DingRtcAudioSessionOperationRestrictionNone              = 0,
    /** SDK不能修改AVAudioSession的Category。 */
    DingRtcAudioSessionOperationRestrictionSetCategory       = 1 << 0,
    /** SDK不能修改AVAudioSession的配置，包括Category，Mode，CategoryOptions。 */
    DingRtcAudioSessionOperationRestrictionConfigureSession  = 1 << 1,
    /** SDK不能关闭AVAudioSession的活动状态，离开频道时，AVAudioSession依然处于活动状态。 */
    DingRtcAudioSessionOperationRestrictionDeactivateSession = 1 << 2,
};
#endif


/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 相机流类型。
 
 - DingRtcVideoStreamTypeNone：无，在OnSubscribeStreamTypeChanged回调表示当前未订阅
 - DingRtcVideoStreamTypeFHD：超高清流（设备性能、网络状态、推流分辨率符合要求情况下满足），对应1080P收费档
 - DingRtcVideoStreamTypeHD：高清流（设备性能、网络状态、推流分辨率符合要求情况下满足），对应720P收费档
 - DingRtcVideoStreamTypeSD：标清（设备性能、网络状态、推流分辨率符合要求情况下满足），对应480P收费档
 - DingRtcVideoStreamTypeLD：流畅（设备性能、网络状态、推流分辨率符合要求情况下满足），对应480P收费档
 */
typedef NS_ENUM(NSInteger, DingRtcVideoStreamType) {
    /** 无，在OnSubscribeStreamTypeChanged回调表示当前未订阅。 */
    DingRtcVideoStreamTypeNone = 0,
    /** 超高清流（设备性能、网络状态、推流分辨率符合要求情况下满足），对应1080P收费档。 */
    DingRtcVideoStreamTypeFHD = 1,
    /** 高清流（设备性能、网络状态、推流分辨率符合要求情况下满足），对应720P收费档。 */
    DingRtcVideoStreamTypeHD = 2,
    /** 标清（设备性能、网络状态、推流分辨率符合要求情况下满足），对应480P收费档。 */
    DingRtcVideoStreamTypeSD = 3,
    /** 流畅（设备性能、网络状态、推流分辨率符合要求情况下满足），对应480P收费档。 */
    DingRtcVideoStreamTypeLD = 4,
};

#if TARGET_OS_IPHONE
/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @remark ios
 * @brief 设备音量类型 (仅适用于 iOS 系统)。
 * @note 默认为Auto类型，SDK 内部会自动选择合适的音量类型，如无特殊需求，不推荐自行设置。
 
 - DingRtcDeviceVolumeTypeAuto：SDK自动设置音量类型
 - DingRtcDeviceVolumeTypeMedia：设置为媒体音量，适合媒体播放场景，音量可以调节到零
 - DingRtcDeviceVolumeTypeVOIP：设置为通话音量，适合通话场景，音量无法调节到零
 
 */
typedef NS_ENUM(NSInteger, DingRtcDeviceVolumeType) {
    /** SDK自动设置音量类型。 */
    DingRtcDeviceVolumeTypeAuto = 0,
    /** 设置为媒体音量，适合媒体播放场景，音量可以调节到零。 */
    DingRtcDeviceVolumeTypeMedia = 1,
    /** 设置为通话音量，适合通话场景，音量无法调节到零。 */
    DingRtcDeviceVolumeTypeVOIP = 2,
};
#endif

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 视频裸数据源类型。

 - DingRtcVideosourceCameraType：视频流
 - DingRtcVideosourceScreenShareType：屏幕共享流
 - DingRtcVideosourceBoth：For video record
 - DingRtcVideosourceTypeMax：占位符，无意义
 */
typedef NS_ENUM(NSInteger, DingRtcVideoSource) {
    /** 视频流。 */
    DingRtcVideosourceCameraType = 0,
    /** 屏幕共享流。 */
    DingRtcVideosourceScreenShareType = 1,
#if TARGET_OS_MAC && !TARGET_OS_IPHONE
    /** For video record。 */
    DingRtcVideosourceBoth = 2,
#endif
    /** 占位符，无意义。 */
    DingRtcVideosourceTypeMax,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 音频数据源类型。
 */
typedef NS_ENUM(NSInteger, DingRtcAudioSource) {
    /*! 采集的音频数据。 */
    DingRtcAudioSourceCaptured = 0,
    /*! 3A后的音频数据。 */
    DingRtcAudioSourceProcessCaptured = 1,
    /*! 推流的音频数据。 */
    DingRtcAudioSourcePub = 2,
    /*! 播放的音频数据。 */
    DingRtcAudioSourcePlayback = 3,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 视频数据格式。
 */
typedef NS_ENUM(NSInteger, DingRtcVideoFormat) {
    DingRtcVideoFormat_UNKNOW = -1,
    DingRtcVideoFormat_BGRA = 0,
    DingRtcVideoFormat_I420,
    DingRtcVideoFormat_NV21,
    DingRtcVideoFormat_NV12,
    DingRtcVideoFormat_RGBA,
    DingRtcVideoFormat_I422,
    DingRtcVideoFormat_ARGB,
    DingRtcVideoFormat_ABGR,
    DingRtcVideoFormat_RGB24,
    DingRtcVideoFormat_BGR24,
    DingRtcVideoFormat_RGB565,
    DingRtcVideoFormat_H264,
};


/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 视频数据类型。

 - DingRtcBufferType_Raw_Data：YUV裸数据
 - DingRtcBufferType_CVPixelBuffer：CVPixelBuffer数据
 */
typedef NS_ENUM(NSInteger, DingRtcBufferType) {
    /** YUV裸数据。 */
    DingRtcBufferType_Raw_Data = 0,
    /** CVPixelBuffer数据。 */
    DingRtcBufferType_CVPixelBuffer,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 音频声道数。
 */
typedef NS_ENUM(NSInteger, DingRtcAudioNumChannel) {
    /** 单声道。 */
    DingRtcMonoAudio = 1,
    /** 双声道。 */
    DingRtcStereoAudio = 2,
};

#if TARGET_OS_IPHONE
/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @remark ios
 * @brief 文件录制质量 (仅适用于 iOS 系统)。
 
 - DingRtcRecording_Low_Quality：低质量文件录制，文件体积较小，音质一般
 - DingRtcRecording_Midium_Quality：中等质量文件录制，文件体积中等，音质中等
 - DingRtcRecording_High_Quality：高质量文件录制，文件体积较大，音质较好
 */
typedef NS_ENUM(NSInteger, DingRtcRecordingQuality) {
    DingRtcRecording_Low_Quality = 0,
    DingRtcRecording_Midium_Quality = 1,
    DingRtcRecording_High_Quality = 2,
};
#endif

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 变声音效模式。
 */
typedef NS_ENUM(NSInteger, DingRtcAudioEffectVoiceChangerMode) {
    /** 关闭变声音效。 */
    DingRtcAudioEffectvVoiceChanger_OFF = 0,
    /** 老人。 */
    DingRtcAudioEffectvVoiceChanger_OLD_MAN,
    /** 男孩。 */
    DingRtcAudioEffectvVoiceChanger_BABYBOY,
    /** 女孩。 */
    DingRtcAudioEffectvVoiceChanger_BABYGILR,
    /** 机器人。 */
    DingRtcAudioEffectvVoiceChanger_ROBOT,
    /** 大魔王。 */
    DingRtcAudioEffectvVoiceChanger_DAIMO,
    /** KTV。 */
    DingRtcAudioEffectvVoiceChanger_KTV,
    /** 回声。 */
    DingRtcAudioEffectvVoiceChangerR_ECHO,
    /** 占位符。 */
    DingRtcAudioEffectvVoiceChanger_MAX,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 音效混响模式。
 */
typedef NS_ENUM(NSInteger, DingRtcAudioEffectReverbMode) {
    /** 关闭混响。 */
    DingRtcAudioEffectReverb_Off = 0,
    /** 人声I。 */
    DingRtcAudioEffectReverb_Vocal_I,
    /** 人声II。 */
    DingRtcAudioEffectReverb_Vocal_II,
    /** 澡堂。 */
    DingRtcAudioEffectReverb_Bathroom,
    /** 明亮小房间。 */
    DingRtcAudioEffectReverb_Small_Room_Bright,
    /** 黑暗小房间。 */
    DingRtcAudioEffectReverb_Small_Room_Dark,
    /** 中等房间。 */
    DingRtcAudioEffectReverb_Medium_Room,
    /** 大房间。 */
    DingRtcAudioEffectReverb_Large_Room,
    /** 教堂走廊。 */
    DingRtcAudioEffectReverb_Church_Hall,
    /** 占位符。 */
    DingRtcAudioEffectReverb_Mode_Max,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 音效混响参数。
 */
typedef NS_ENUM(NSInteger, DingRtcAudioEffectReverbParamType) {
    /** 房间大小，取值范围：[0, 100]。 */
    DingRtcAudioEffectReverb_Room_Size = 0,
    /** 预延时，单位ms，取值范围：[0, 200]。 */
    DingRtcAudioEffectReverb_Pre_Delay,
    /** 混响感，取值范围：[0, 100]。 */
    DingRtcAudioEffectReverb_Revetberance,
    /** 消声，取值范围：[0, 100]。 */
    DingRtcAudioEffectReverb_Hf_Damping,
    /** 低音调，取值范围：[0, 100]。 */
    DingRtcAudioEffectReverb_Tone_Low,
    /** 高音调，取值范围：[0, 100]。 */
    DingRtcAudioEffectReverb_Tone_High,
    /** 干增益，取值范围：[-20, 10]。 */
    DingRtcAudioEffectReverb_Dry_Gain,
    /** 湿增益，取值范围：[-20, 10]。 */
    DingRtcAudioEffectReverb_Wet_Gain,
    /** 占位符。 */
    DingRtcAudioEffectReverb_Type_Max,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 网络连接状态。
 */
typedef NS_ENUM(NSInteger, DingRtcConnectionStatus) {
    /** 初始化完成。 */
    DingRtcConnectionStatusInit = 0,
    /** 网络连接断开。 */
    DingRtcConnectionStatusDisconnected,
    /** 建立网络连接中。 */
    DingRtcConnectionStatusConnecting,
    /** 网络已连接。 */
    DingRtcConnectionStatusConnected,
    /** 重新建立网络连接中。 */
    DingRtcConnectionStatusReconnecting,
    /** 网络连接失败。 */
    DingRtcConnectionStatusFailed,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 引起网络状态变更原因。
 */
typedef NS_ENUM(NSInteger, DingRtcConnectionStatusChangeReason) {
    /** 未知原因。 */
    DingRtcConnectionChangedDummyReason = 0,
    /** 心跳超时。 */
    DingRtcConnectionChangedSignalingHeartbeatTimeout,
    /** 心跳恢复。 */
    DingRtcConnectionChangedSignalingHeartbeatDingve,
    /** 加入频道失败。 */
    DingRtcConnectionChangedSignalingJoinChannelFailure,
    /** 加入频道成功。 */
    DingRtcConnectionChangedSignalingJoinChannelSuccess,
    /** 离开频道。 */
    DingRtcConnectionChangedSignalingLeaveRoom,
    /** 信令建立连接。 */
    DingRtcConnectionChangedSignalingConnecting,
    /** 网络连接中断。 */
    DingRtcConnectionChangedNetworkInterrupted,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 频道类型。
 */
typedef NS_ENUM(NSInteger, DingRtcChannelProfile) {
    /** 通信模式。 */
    DingRtcCommunication = 0,
    /** 互动模式。 */
    DingRtcInteractivelive,
    /** 低延时互动直播模式。 */
    DingRtcInteractiveWithLowLatencyLive,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 角色类型。
 */
typedef NS_ENUM(NSInteger, DingRtcClientRole) {
    /** 互动角色。 */
    DingRtcClientRoleInteractive = 0,
    /** 观众角色。 */
    DingRtcClientRolelive,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 本地静音模式。

- DingRtcMuteAudioModeDefault：默认模式(静音全部，包括麦克风及外部输入音频)
- DingRtcMuteAllAudioMode：静音全部(包括麦克风及外部输入音频)
- DingRtcMuteOnlyMicAudioMode：只静音麦克风
*/
typedef NS_ENUM(NSInteger, DingRtcMuteLocalAudioMode) {
    /** 默认模式(静音全部，包括麦克风及外部输入音频)。 */
    DingRtcMuteAudioModeDefault = 0,
    /** 静音全部(包括麦克风及外部输入音频)。 */
    DingRtcMuteAllAudioMode,
    /** 只静音麦克风。 */
    DingRtcMuteOnlyMicAudioMode,
    DingRtcMuteLocalAudioMax,
};


/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 镜像模式。
 */
typedef NS_ENUM(NSInteger, DingRtcRenderMirrorMode) {
#if TARGET_OS_IPHONE
    /** 只有前置摄像头预览镜像，其余不镜像 (仅适用于 iOS 系统)。 */
    DingRtcRenderMirrorModeOnlyFrontCameraPreviewEnabled = 0,
#endif
    /** 镜像开启。 */
    DingRtcRenderMirrorModeAllEnabled = 1,
    /** 镜像关闭。 */
    DingRtcRenderMirrorModeAllDisabled = 2,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 音乐伴奏播放状态。
 */
typedef NS_ENUM(NSInteger, DingRtcAudioPlayingStateCode) {
    /** 开始播放。 */
    DingRtcAudioPlayingStarted       = 100,
    /** 停止播放。 */
    DingRtcAudioPlayingStopped       = 101,
    /** 播放暂停。 */
    DingRtcAudioPlayingPaused        = 102,
    /** 播放恢复。 */
    DingRtcAudioPlayingResumed       = 103,
    /** 播放完毕。 */
    DingRtcAudioPlayingEnded         = 104,
    /** 正在缓冲。 */
    DingRtcAudioPlayingBuffering     = 105,
    /** 缓冲结束。 */
    DingRtcAudioPlayingBufferingEnd  = 106,
    /** 播放失败。 */
    DingRtcAudioPlayingFailed        = 107,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 音乐伴奏播放错误码。
 */
typedef NS_ENUM(NSInteger, DingRtcAudioPlayingErrorCode) {
    /** 没有错误。 */
    DingRtcAudioPlayingNoError        = 0,
    /** 打开文件失败。 */
    DingRtcAudioPlayingOpenFailed     = -100,
    /** 解码文件失败。 */
    DingRtcAudioPlayingDecodeFailed   = -101,
};

/**
 * @brief 录制类型。

 - DingRtcRecordTypeAudio：纯音频录制。
*/
typedef NS_ENUM(NSUInteger, DingRtcRecordType) {
    /** 录制单个音频文件。 */
    DingRtcRecordTypeAudio = 0,
#if TARGET_OS_MAC && !TARGET_OS_IPHONE
    /** 录制单个音视频文件 (仅适用于 Mac 系统)。 */
    DingRtcRecordTypeVideo,
    /** 录制纯音频和音视频文件 (仅适用于 Mac 系统)。 */
    DingRtcRecordTypeBoth,
#endif
};

/**
 * @brief 录制格式。
 */
typedef NS_ENUM(NSUInteger, DingRtcRecordFormat) {
    /** 录制AAC格式文件。 */
    DingRtcRecordFormatAAC = 0,
    /** 录制WAV格式文件。 */
    DingRtcRecordFormatWAV,
#if TARGET_OS_MAC && !TARGET_OS_IPHONE
    /** 录制MP4格式文件（仅适用于 Mac 系统）。 */
    DingRtcRecordFormatMP4,
#endif
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 录制音频质量。
 */
typedef NS_ENUM(NSUInteger, DingRtcAudioQuality) {
    /** 低质量文件录制，文件体积较小，音质一般。 */
    DingRtcAudioQualityLow = 0,
    /** 中等质量文件录制，文件体积中等，音质中等。 */
    DingRtcAudioQualityMidium,
    /** 高质量文件录制，文件体积较大，音质较好。 */
    DingRtcAudioQualityHigh,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 录制视频质量。
 */
typedef NS_ENUM(NSUInteger, DingRtcVideoQuality) {
    /**  默认，和采集保持一致。 */
    DingRtcVideoQualityDefault = 0,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 录制音频采样率。
*/
typedef NS_ENUM(NSInteger, DingRtcAudioSampleRate) {
    /** 采样率：8000。 */
    DingRtcAudioSampleRate_8000 = 0,
    /** 采样率：11025。 */
    DingRtcAudioSampleRate_11025 = 1,
    /** 采样率：16000。 */
    DingRtcAudioSampleRate_16000 = 2,
    /** 采样率：22050。 */
    DingRtcAudioSampleRate_22050 = 3,
    /** 采样率：32000。 */
    DingRtcAudioSampleRate_32000 = 4,
    /** 采样率：44100。 */
    DingRtcAudioSampleRate_44100 = 5,
    /** 采样率：48000。 */
    DingRtcAudioSampleRate_48000 = 6,
};

#if TARGET_OS_IPHONE
/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @remark ios
 * @brief 语音路由 (仅适用于 iOS 系统)。
 */
typedef NS_ENUM(NSInteger, DingRtcAudioRouteType) {
    /** 默认的语音路由。 */
    DingRtcAudioRouteTypeDefault        = 0,
    /** 耳机。 */
    DingRtcAudioRouteTypeHeadset        = 1,
    /** 听筒。 */
    DingRtcAudioRouteTypeEarpiece       = 2,
    /** 不带麦克风的耳机。 */
    DingRtcAudioRouteTypeHeadsetNoMic   = 3,
    /** 手机扬声器。 */
    DingRtcAudioRouteTypeSpeakerphone   = 4,
    /** 外接扬声器。 */
    DingRtcAudioRouteTypeLoudSpeaker    = 5,
    /** 蓝牙耳机。 */
    DingRtcAudioRouteTypeBlueTooth      = 6,
};
#endif

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 用户离线原因。
 */
typedef NS_ENUM(NSInteger, DingRtcShowDebugViewType){
    DingRtcShowTypeNone     = 0,  /* 不显示 */
    DingRtcShowTypeAudio    = 1,  /* 音频 */
    DingRtcShowTypeVideo    = 2,  /* 视频 */
    DingRtcShowTypeNetwork  = 3,  /* 网络 */
    DingRtcShowTypeAll      = 4,  /* 全部 */
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 屏幕共享模式 支持位运算按需组合。
 */
typedef NS_ENUM(NSInteger, DingRtcScreenShareMode){
    /** 不共享音视频流。 */
    DingRtcScreenShareNone       = 0,
    /** 共享视频流。 */
    DingRtcScreenShareVideo      = 1 << 0,
    /** 共享App音频流。 */
    DingRtcScreenShareAudioApp   = 1 << 1,
    /** 共享所有音视频流，不支持屏幕共享Mic音频流，请使用SDK自带的Mic音频采集。 */
    DingRtcScreenShareAll        = 3,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 录制音频设置。
 */
typedef struct  {
    /** 音频采样率类型。 */
    DingRtcAudioSampleRate sampleRate;
    /** 音频录制质量。 */
    DingRtcAudioQuality quality;
    /** 是否允许录制外部pcm输入渲染数据。 */
    bool enableRecordExternalRenderPCM;
#if TARGET_OS_MAC && !TARGET_OS_IPHONE
    /** 是否允许录制外部pcm输入采集数据  (仅适用于 Mac 系统)。 */
    bool enableRecordExternalCapturePCM;
#endif
}DingRtcRecordAudioConfig;

#if TARGET_OS_MAC && !TARGET_OS_IPHONE
/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief mac。
 * @brief 录制视频分辨率 (仅适用于 Mac 系统)。
 * @note 画布大小。
 */
typedef struct {
    /** 录制视频宽。 */
    int canvasWidth;
    /** 录制视频高。 */
    int canvasHeight;
} DingRtcRecordVideoCanvasConfig;

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief mac
 * @brief 录制视频坐标 (仅适用于 Mac 系统)。
 */
typedef struct {
    uint32_t numerator;
    uint32_t denominator;
} DingRtcRecordVideoRational;

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief mac。
 * @brief 录制视频位置信息 (仅适用于 Mac 系统)。
 */
typedef struct {
    DingRtcRecordVideoRational left;
    DingRtcRecordVideoRational top;
    DingRtcRecordVideoRational width;
    DingRtcRecordVideoRational height;
} DingRtcRecordVideoRectangle;

#endif

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 录制视频设置。
 */
typedef struct  {
    /** 视频质量。 */
    DingRtcVideoQuality quality;
#if TARGET_OS_MAC && !TARGET_OS_IPHONE
    /** 录制文件类型 (仅适用于 Mac 系统)。 */
    DingRtcRecordType sourceType;
    /** 录制文件尺寸 (仅适用于 Mac 系统)。 */
    DingRtcRecordVideoCanvasConfig canvas;
    /** 录制帧率 (仅适用于 Mac 系统)。 */
    int fps;
    /** 录制码率 (仅适用于 Mac 系统)。 */
    int bitrate;
#endif
} DingRtcRecordVideoConfig;


/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 错误码。
 */
typedef NS_ENUM(NSInteger, DingRtcErrorCode) {
    /** 状态正常，无错误。 */
    DingRtcErrNone                           = 0,

    /****************************************************
     * 入会错误码
     ****************************************************/
    /** AppId不存在，请在控制台创建应用。 */
    DingRtcErrJoinBadAppId                   = 0x02010201,
    /** AppId已失效，请在控制台重新启用应用。 */
    DingRtcErrJoinInvalidAppId               = 0x02010202,
    /**
     * 频道不存在，请尝试重新加入频道。
     * 如会议发起者入会后又很快离会，第二个人随后入会，发现会议已经不存在了。
     */
    DingRtcErrJoinBadChannel                 = 0x02010204,
    /** 频道已经失效（目前频道有效期为48小时），请重新生成频道鉴权令牌（Token）。 */
    DingRtcErrJoinInvalidChannel             = 0x02010203,
    /** Token无效。 请重新生成频道鉴权令牌（Token。） */
    DingRtcErrJoinBadToken                   = 0x02010205,
    /** 加入频道超时，请检查网络连接是否正常。 */
    DingRtcErrJoinTimeout                    = 0x01020204,
    /** 参数错误，{@link DingRtcAuthInfo}中的字段为空或者timestamp<=0，请检查字段是否完整。 */
    DingRtcErrJoinBadParam                   = 0x01030101,
    /** 加入频道失败。 */
    DingRtcErrJoinChannelFailed              = 0x01030202,

    /****************************************************
     * 日志模块错误码
     ****************************************************/
    /** 日志模块已经初始化。 */
    DingRtcErrLogAlreadyInit = 0x01030105,
    /** 日志模块根路径错误，请检查路径是否存在或路径是否具有写入权限。 */
    DingRtcErrLogRootPathError = 0x01030106,

    /****************************************************
     * 音频设备错误码
     ****************************************************/
    /**
     * 采集设备初始化失败、采集设备媒体服务异常、音频设备被占用。
     * - Mac端和Windows端查看系统设置中音频设备是否有能量条。
     * - iOS端和Android端确认是否被其他应用占用。
     * - 如果都正常，请重启设备。
     */
    DingRtcErrMicOpenFail                    = 0x01040404,
    /**
     * 播放设备初始化失败、播放设备媒体服务异常、音频设备被占用。
     * - Mac端和Windows端查看系统设置中音频设备是否有能量条。
     * - iOS端和Android端确认是否被其他应用占用。
     * - 如果都正常，请重启设备。
     */
    DingRtcErrSpeakerOpenFail                = 0x01040405,
    /** 系统性能不足、采集设备媒体服务异常、音频设备被占用。建议重启应用或重启设备。 */
    DingRtcErrMicInterrupt                   = 0x01040406,
    /** 系统性能不足，播放设备媒体服务异常，音频设备被占用。建议重启应用或重启设备。 */
    DingRtcErrSpeakerInterrupt               = 0x01040407,
    /** 麦克风设备未授权，建议去系统设置中，打开麦克风授权，并且重新加入频道。 */
    DingRtcErrMicAuthFail                    = 0x01040408,
    /** 无可用的音频采集设备，检查是否接入了可用的麦克风，并检查系统设置中，该设备是否被禁用。 */
    DingRtcErrMicNotAvailable                = 0x01040409,
    /** 无可用的音频播放设备，检查是否接入了可用的扬声器或耳机，并检查系统设置中，该设备是否被禁用。 */
    DingRtcErrSpeakerNotAvailable            = 0x01040410,

    /****************************************************
     * 视频设备错误码
     ****************************************************/
    /** 采集设备初始化失败，检查系统相机是否可用，移动端App是否占用了相机。 */
    DingRtcErrCameraOpenFail                 = 0x01040104,
    /** 采集过程中出现异常，Mac、Winodows电脑相机采集灯熄灭，设备被强制中断。建议重新加入频道或者重启App。 */
    DingRtcErrCameraInterrupt                = 0x01040106,
    /** 渲染设备初始化失败，检查Windows端显卡驱动是否需要升级，移动端手机型号或者Android平板盒子等外设是否支持OpenGL。 */
    DingRtcErrDisplayOpenFail                = 0x01040201,
    /** 渲染过程中出现异常，系统异常错误导致渲染引擎报错，重新加入频道或者重启App。 */
    DingRtcErrDisplayInterrupt               = 0x01040202,

    /****************************************************
     * 网络错误码
     ****************************************************/
    /** 媒体通道建立失败，检查网络连接是否正常。 */
    DingRtcErrIceConnectionConnectFail       = 0x01050201,
    /** 媒体通道重连失败，检查网络连接是否正常。 */
    DingRtcErrIceConnectionReconnectFail     = 0x01050202,
    /** 信令心跳超时，检查网络连接是否正常。 */
    DingRtcErrConnectionHeartbeatTimeout     = 0x0102020C,

    /****************************************************
     * 低延时互动直播模式相关错误码
     ****************************************************/
    /** 低延时互动直播信令失败。 */
    DingRtcErrCdnLiveCommunicationFailed     = 0x01070003,
    /** 低延时互动直播媒体设置错误。 */
    DingRtcErrCdnLiveMeidaSettingFailed      = 0x01070004,
    /** 低延时互动直播当前房间没有主播推流。 */
    DingRtcErrCdnLiveNoneBroadcaster         = 0x01070005,
    /** 低延时互动直播错误，应用层需要重新创建新的SDK实例。 */
    DingRtcErrCdnLiveNeedRestart             = 0x01070006,
    /** 低延时互动直播错误，应用层需要稍等一定时间（自定义间隔）重新创建新的SDK实例。 */
    DingRtcErrCdnLiveNeedRestartAgainLater   = 0x01070007,
    /** 低延迟互动直播服务不可用或者服务降级，客户端应该选择其它方式拉超大方会流，即不从cdn sfu拉流。 */
    DingRtcErrCdnLiveServiceUnavailable      = 0x01070008,
    /** 低延迟互动直播网络中断错误，销毁当前sdk实例重新创建sdk后重新走订阅流程，已废弃。 */
    DingRtcErrCdnLiveMediaConnectionInterruption = 0x01070009,

    /****************************************************
     * 推流相关错误码
     ****************************************************/
    /** 推流无效。 */
    DingRtcErrPublishInvaild                 = 0x01030305,
    /** 未进入频道推流失败。 */
    DingRtcErrPublishNotJoinChannel          = 0x01010406,
    /** 推送音频流失败。 */
    DingRtcErrPublishAudioStreamFailed       = 0x01010450,
    /** 推送视频流失败。 */
    DingRtcErrPublishVideoStreamFailed       = 0x01010451,
    /** 推送屏幕共享失败。 */
    DingRtcErrPublishScreenShareFailed       = 0x01010453,
    /** 屏幕共享配置错误。 */
    DingRtcErrPublishScreenShareConfigError  = 0x01010454,

    /****************************************************
     * 订阅相关错误码
     ****************************************************/
    /** 订阅无效。 */
    DingRtcErrSubscribeInvaild               = 0x01030404,
    /** 未进入频道订阅错误。 */
    DingRtcErrSubscribeNotJoinChannel        = 0x01010550,
    /** 订阅音频流失败。 */
    DingRtcErrSubscribeAudioStreamFailed     = 0x01010551,
    /** 订阅视频流失败。 */
    DingRtcErrSubscribeVideoStreamFailed     = 0x01010552,
    /** 订阅屏幕共享失败。 */
    DingRtcErrSubscribeScreenShareFailed     = 0x01010554,

    /****************************************************
     * 其他错误码
     ****************************************************/
    /** SDK状态错误，建议销毁SDK重新创建实例。 */
    DingRtcErrSdkInvalidState                = 0x01030204,
    /** 参数不匹配。 */
    DingRtcErrInvalidArguments               = 0x01030103,
    /** Session已经被移除，建议销毁实例之后重新创建实例并加入频道，已废弃。 */
    DingRtcErrSessionRemoved                 = 0x02010105,
    /** buffer队列饱和，用于外部输送裸数据功能，建议降低外部输入频次，设置等待20ms，并且单次不超过40ms数据。 */
    DingRtcErrAudioBufferFull                = 0x01070101,
    /** buffer队列饱和，用于外部输送裸数据功能，建议降低外部输入频次，设置等待视频帧间隔后再输入。 */
    DingRtcErrVideoBufferFull                = 0x01070102,
    /** 互动模式下设置角色错误。 */
    DingRtcErrUpdateRoleChannel              = 0x01020201,
    /** SDK内部错误。 */
    DingRtcErrInner                          = -1,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief OnBye类型。
 */
typedef NS_ENUM(NSInteger, DingRtcOnByeType) {
    /** 当前user被踢出channel。 */
    DingRtcOnByeBeKickedOut          = 1,
    /** channel已结束，需要离开会议。 */
    DingRtcOnByeChannelTerminated    = 2,
    /** 相同userID在其他设备joinChannel，当前设备被下线。 */
    DingRtcOnByeUserReplaced         = 3,
    /** 信令服务失败，自动重连失败，需要用户手动重试。 */
    DingRtcOnByeNeedRelogin          = 4,
};

/**
 * @brief SDK问题反馈类型。
 */
typedef NS_ENUM(NSInteger, DingRtcFeedbackType) {
    /** 视频设备问题。 */
    DingRtcFeedbackVideoDeviceFail   = 101,
    /** 视频无画面。 */
    DingRtcFeedbackVideoNotRender    = 102,
    /** 音频设备问题。 */
    DingRtcFeedbackAudioDeviceFail   = 201,
    /** 音频无声音。 */
    DingRtcFeedbackAudioNotRender    = 202,
    /** 音频回声异常。 */
    DingRtcFeedbackAudioEchoError    = 203,
    /** 会议状态异常：入会异常、离会异常等。 */
    DingRtcFeedbackChannelTypeError  = 301,
    /** SDK其他状态异常。 */
    DingRtcFeedbackSDKTypeError      = 302,
    /** 网络卡顿问题。 */
    DingRtcFeedbackNetworkUnfluent   = 401,
    /** 视频清晰问题。 */
    DingRtcFeedbackVideoBlurring     = 402,
    /** 未知类型。 */
    DingRtcFeedbackUnkonw            = -1,
};

/**
 * @brief 旁路推流LiveStream状态。
 */
typedef NS_ENUM(NSInteger, DingRtcTrascodingLiveTaskStatus) {
    /** 任务开始。 */
    DingRtcTrascodingLiveTaskStatusStart = 0,
    /** 任务更新。 */
    DingRtcTrascodingLiveTaskStatusUpdate = 1,
    /** 任务已停止。 */
    DingRtcTrascodingLiveTaskStatusStop = 2,
};

/**
 * @brief 旁路推流错误码。
 */
typedef NS_ENUM(NSInteger, DingRtcTrascodingLiveStreamErrorCode) {
    DingRtcTrascodingLiveStreamErrorPublishOk            = 0,
    DingRtcTrascodingLiveStreamErrorStreamNotFound       = 0x01100001,
    DingRtcTrascodingLiveStreamErrorStreamAlreadyExist   = 0x01100002,
    DingRtcTrascodingLiveStreamErrorInvalidParam         = 0x01100003,
    DingRtcTrascodingLiveStreamErrorInternalError        = 0x01100004,
    DingRtcTrascodingLiveStreamErrorRtmpServerError      = 0x01100005,
    DingRtcTrascodingLiveStreamErrorRtmpStreamUrlError   = 0x01100006,
    DingRtcTrascodingLiveStreamErrorPublishTimeout       = 0x01100007,
    DingRtcTrascodingLiveStreamErrorNotAuthorized        = 0x01100008,
};

/**
 * @brief 跨频道转推事件。
 */
typedef NS_ENUM(NSInteger, DingRtcChannelRelayEvent) {
    DingRtcChannelRelayStart     = 0,
    DingRtcChannelRelayUpdate    = 1,
    DingRtcChannelRelayStop      = 2,
};

/**
 * @brief 跨频道转推事件。
 */
typedef NS_ENUM(NSInteger, DingRtcChannelRelayState) {
    DingRtcChannelRelayStateInit         = 3,
    DingRtcChannelRelayStateConnecting   = 4,
    DingRtcChannelRelayStateRunning      = 5,
    DingRtcChannelRelayStateFailure      = 6,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 采集偏好。
*/
typedef NS_ENUM(NSInteger, DingRtcCaptureOutputPreference)  {
  /** SDK自动调整采集参数。 */
  DingRtcCaptureOutputPreferenceAuto = 0, 
  /** SDK优先保证设备性能，根据设置编码参数选择最接近的摄像头输出参数。 */
  DingRtcCaptureOutputPreferencePerformance = 1,
  /** SDK优先保证视频预览质量，选择较高的摄像头输出参数。 */
  DingRtcCaptureOutputPreferencePreview = 2, 
};

#if TARGET_OS_IPHONE
/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @remark ios
 * @brief 摄像头方向 (仅适用于 iOS 系统)。
 */
typedef NS_ENUM(NSInteger, DingRtcCameraDirection)  {
  DingRtcCameraDirectionInvalid = -1,
  DingRtcCameraDirectionBack = 0,
  DingRtcCameraDirectionFront = 1,
};
#endif

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 推流状态。
*/
typedef NS_ENUM(NSInteger, DingRtcPublishState) {
    /** 0：SDK初始推流状态。
     */
    DingRtcStatsPublishIdle = 0,
    /** 1：未推流状态，可能是因为:
     * - 已设置停止推送媒体流。
     * - 推送媒体流失败。
     */
    DingRtcStatsNoPublish   = 1,
    /** 2：推流链接建立过程中。
     */
    DingRtcStatsPublishing  = 2,
    /** 3：推流成功。
     */
    DingRtcStatsPublished   = 3,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 订阅状态。
 
 - DingRtcStatsSubscribeIdle:初始状态
 - DingRtcStatsNoSubscribe：未订阅
 - DingRtcStatsSubscribing：订阅中
 - DingRtcStatsSubscribed： 已订阅
*/
typedef NS_ENUM(NSInteger, DingRtcSubscribeState) {
    /** 初始状态。
     */
    DingRtcStatsSubscribeIdle = 0,
    /** 未订阅。
     */
    DingRtcStatsNoSubscribe   = 1,
    /** 订阅中。
     */
    DingRtcStatsSubscribing   = 2,
    /** 已订阅。
     */
    DingRtcStatsSubscribed    = 3,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 用户离线原因。
*/
typedef NS_ENUM(NSInteger, DingRtcUserOfflineReason) {
    /** 用户主动离开。 */
    DingRtcUserOfflineQuit = 0,
    /** 因过长时间收不到对方数据包，超时掉线。 */
    DingRtcUserOfflineDropped = 1,
    /** 用户身份从主播切换为观众时触发。 */
    DingRtcUserOfflineBecomeAudience = 2,
};

/**
 * @brief 背景替换背景图缩放模式。

- DingRtcBokehScaleModelCrop：等比裁剪
- DingRtcBokehScaleModelFill：填充黑边
*/
typedef NS_ENUM(NSUInteger, DingRtcBokehScaleModel) {
    /** 等比裁剪。 */
    DingRtcBokehScaleModelCrop = 0,
    /** 填充黑边。 */
    DingRtcBokehScaleModelFill = 1,
};

#if TARGET_OS_MAC && !TARGET_OS_IPHONE
/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @remark mac
 * @brief 桌面分享类型 (仅适用于 Mac 系统)。
 */
typedef NS_ENUM(NSInteger, DingRtcScreenShareType){
    /** 桌面分享。 */
    DingRtcScreenShareDesktop = 0,
    /** 窗口分享。 */
    DingRtcScreenShareWindow  = 1,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @remark mac
 * @brief 外接设备类型 (仅适用于 Mac 系统)。
 */
typedef NS_ENUM(NSInteger, DingRtcExternalDeviceType) {
    DingRtcExternalTypeUnknown = -1,       /* 未知类型 */
    DingRtcExternalTypeAudioCapture = 0,   /* 音频采集设备 */
    DingRtcExternalTypeAudioPlayout = 1,   /* 音频播放设备 */
    DingRtcExternalTypeVideoCapture = 2,   /* 视频采集设备 */
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @remark mac
 * @brief 外接设备状态 (仅适用于 Mac 系统)。
 */
typedef NS_ENUM(NSInteger, DingRtcExternalDeviceState) {
    DingRtcExternalDeviceAdd = 0,      /* 新增外接设备 */
    DingRtcExternalDeviceRemove = 1,   /* 移除外接设备 */
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @remark mac
 * @brief 设备传输类型 (仅适用于 Mac 系统)。
 */
typedef NS_ENUM(NSInteger, DingRtcDeviceTransportType) {
    DingRtcDeviceTransportTypeUnknown      = 0,
    DingRtcDeviceTransportTypeBuiltIn      = 1,
    DingRtcDeviceTransportTypeBluetooth    = 2,
    DingRtcDeviceTransportTypeUSB          = 3,
    DingRtcDeviceTransportTypeAggregate    = 4,
    DingRtcDeviceTransportTypeVirtual      = 5,
    DingRtcDeviceTransportTypePCI          = 6,
    DingRtcDeviceTransportTypeFireWire     = 7,
    DingRtcDeviceTransportTypeBluetoothLE  = 8,
    DingRtcDeviceTransportTypeHDMI         = 9,
    DingRtcDeviceTransportTypeDisplayPort  = 10,
    DingRtcDeviceTransportTypeAirPlay      = 11,
    DingRtcDeviceTransportTypeAVB          = 12,
    DingRtcDeviceTransportTypeThunderbolt  = 13
};

#endif

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 质量报告数据。
*/
@interface DingRtcQualityReport : NSObject

// cpu
@property (nonatomic, assign) int cpu_physical_cores;
@property (nonatomic, assign) int cpu_logic_cores;
@property (nonatomic, assign) int cpu_frequency_mhz;
@property (nonatomic, assign) int cpu_used_by_app; // percentage
@property (nonatomic, assign) int cpu_used_by_other_apps; // percentage

// memory
@property (nonatomic, assign) int memory_total_mb;
@property (nonatomic, assign) int memory_used_by_app;
@property (nonatomic, assign) int memory_used_by_other_apps;

// network
/** 丢包率。 */
@property (nonatomic, assign) int loss_rate; // percentage
/** 延时。 */
@property (nonatomic, assign) int rtt_ms;
/** 接收码率。 */
@property (nonatomic, assign) int recv_bitrate;
/** 发送码率。 */
@property (nonatomic, assign) int send_bitrate;
@property (nonatomic, assign) int signal_level;
/** 网络类型。 */
@property (nonatomic, copy) NSString * _Nonnull network_type;

// audio up
/** 音频发送码率。 */
@property (nonatomic, assign) int audio_send_bitrate;
/** 音频发送音量。 */
@property (nonatomic, assign) int audio_send_volume; // in db

// audio down
/** 音频接收码率。 */
@property (nonatomic, assign) int audio_recv_bitrate;
/** 音频接收音量。 */
@property (nonatomic, assign) int audio_recv_volume; // in db

// video up
/** 视频发送码率。 */
@property (nonatomic, assign) int camera_send_bitrate;
/** 视频发送的采集帧率。 */
@property (nonatomic, assign) int camera_send_capture_fps;
@property (nonatomic, assign) int camera_send_drop_fps;
/** 视频发送帧率。 */
@property (nonatomic, assign) int camera_send_fps;
@property (nonatomic, assign) int camera_send_hw;
/** 视频发送宽度。 */
@property (nonatomic, assign) int camera_send_width;
/** 视频发送高度。 */
@property (nonatomic, assign) int camera_send_height;

/** 屏幕共享发送码率。 */
@property (nonatomic, assign) int screenshare_send_bitrate;
/** 屏幕共享采集帧率。 */
@property (nonatomic, assign) int screenshare_send_capture_fps;
@property (nonatomic, assign) int screenshare_send_drop_fps;
/** 屏幕共享发送帧率。 */
@property (nonatomic, assign) int screenshare_send_fps;
@property (nonatomic, assign) int screenshare_send_hw;
/** 屏幕共享宽度。 */
@property (nonatomic, assign) int screenshare_send_width;
/** 屏幕共享高度。 */
@property (nonatomic, assign) int screenshare_send_height;

// video down
/** 视频接收码率。 */
@property (nonatomic, assign) int camera_recv_bitrate; // camera stream, recv bitrate
/** 视频接收帧率。 */
@property (nonatomic, assign) int camera_recv_fps;
/** 视频渲染器帧率。 */
@property (nonatomic, assign) int camera_recv_render_fps;
@property (nonatomic, assign) int camera_recv_hw;
/** 视频接收宽度。 */
@property (nonatomic, assign) int camera_recv_width;
/** 视频接收高度。 */
@property (nonatomic, assign) int camera_recv_height;

/** 屏幕共享接收码率。 */
@property (nonatomic, assign) int screenshare_recv_bitrate; // screen stream, recv bitrate
/** 屏幕共享接收帧率。 */
@property (nonatomic, assign) int screenshare_recv_fps;
/** 屏幕共享渲染帧率。 */
@property (nonatomic, assign) int screenshare_recv_render_fps;
@property (nonatomic, assign) int screenshare_recv_hw;
/** 屏幕共享接收宽度。 */
@property (nonatomic, assign) int screenshare_recv_width;
/** 屏幕共享接收高度。 */
@property (nonatomic, assign) int screenshare_recv_height;

@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 基础统计信息。
 */
@interface DingRtcStats : NSObject
/** 可用带宽（kb）。 */
@property (nonatomic, assign) long long available_sent_kbitrate; 
/** 总发送码率（kb）。 */
@property (nonatomic, assign) long long sent_kbitrate;
/** 总接收码率（kb）。 */
@property (nonatomic, assign) long long recv_kbitrate;
/** 总发送数据量（bytes）。 */
@property (nonatomic, assign) long long sent_bytes;
/** 总接收数据量（bytes）。 */
@property (nonatomic, assign) long long recv_bytes;
/** 视频接收码率（kb）。 */
@property (nonatomic, assign) long long video_recv_kbitrate;
/** 视频发送码率（kb）。 */
@property (nonatomic, assign) long long video_sent_kbitrate;
/** 音频接收码率（kb）。 */
@property (nonatomic, assign) long long audio_recv_kbitrate;
/** 音频发送码率（kb）。 */
@property (nonatomic, assign) long long audio_sent_kbitrate;
/** 通话时长(s)。 */
@property (nonatomic, assign) long long call_duration;
/** 进程CPU使用量 。*/
@property (nonatomic, assign) unsigned int cpu_usage;
/** 系统CPU使用量。 */
@property (nonatomic, assign) unsigned int systemCpuUsage;
/** 客户端到服务器的丢包率。 */
@property (nonatomic, assign) long long sent_loss_rate;
/** 客户端到服务器的丢包数。 */
@property (nonatomic, assign) long long sent_loss_pkts;
/** 客户端到服务器的总包数。 */
@property (nonatomic, assign) long long sent_expected_pkts;
/** 客户端到服务器的下行丢包率。 */
@property (nonatomic, assign) long long recv_loss_rate;
/** 客户端到服务器的下行丢包数。 */
@property (nonatomic, assign) long long recv_loss_pkts;
/** 客户端到服务器的下行总包数。 */
@property (nonatomic, assign) long long recv_expected_pkts;
/** 客户端到服务器的延迟（ms）。 */
@property (nonatomic, assign) long long lastmile_delay;

@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 本地视频统计信息。
 */
@interface DingRtcLocalVideoStats : NSObject

/** 视频流track类型。 */
@property (nonatomic, assign) DingRtcVideoTrack track;
/** 目标编码码率。 */
@property (nonatomic, assign) unsigned int targetEncodeBitrate;
/** 实际编码码率。 */
@property (nonatomic, assign) unsigned int actualEncodeBitrate;
/** 发送码率。 */
@property (nonatomic, assign) unsigned int sentBitrate;
/** 采集帧率。 */
@property (nonatomic, assign) unsigned int captureFps;
/** 发送帧率。 */
@property (nonatomic, assign) unsigned int sentFps;
/** 编码器输出帧率。 */
@property (nonatomic, assign) unsigned int encodeFps;
/** 每秒平均QP。 */
@property (nonatomic, assign) unsigned int avgQpPerSec;
/** 编码帧宽。 */
@property (nonatomic, assign) unsigned int encoderFrameWidth;
/** 编码帧高。 */
@property (nonatomic, assign) unsigned int encoderFrameHeight;
/** 采集帧宽。 */
@property (nonatomic, assign) unsigned int captureFrameWidth;
/** 采集帧高。 */
@property (nonatomic, assign) unsigned int captureFrameHeight;

@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 远端视频统计信息。
 */
@interface DingRtcRemoteVideoStats : NSObject
/** 远端用户userID。 */
@property (nonatomic, copy) NSString * _Nonnull userId;
/** 视频流track类型。 */
@property (nonatomic, assign) DingRtcVideoTrack track;
/** 视频帧的宽。 */
@property (nonatomic, assign) unsigned int width;
/** 视频帧的高。 */
@property (nonatomic, assign) unsigned int height;
/** 接收码率。 */
@property (nonatomic, assign) unsigned int recvBitrate;
/** 解码帧率。 */
@property (nonatomic, assign) unsigned int decoderOutputFrameRate;
/** 渲染帧率。 */
@property (nonatomic, assign) unsigned int rendererOutputFrameRate;
/** 丢包率。 */
@property (nonatomic, assign) unsigned int packetLossRate;

@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 本地音频统计信息。
 */
@interface DingRtcLocalAudioStats : NSObject

/** 音频流track类型。 */
@property (nonatomic, assign) DingRtcAudioTrack track;
/** 声道数。 */
@property (nonatomic, assign) unsigned int numChannel;
/** 采样率 */
@property (nonatomic, assign) unsigned int sentSamplerate;
/** 发送端音频的瞬时码率。 */
@property (nonatomic, assign) unsigned int sentBitrate;
/** 输入音量。 */
@property (nonatomic, assign) unsigned int inputLevel;

@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 远端音频统计信息。
 */
@interface DingRtcRemoteAudioStats : NSObject

/** 远端用户userID。 */
@property (nonatomic, copy) NSString * _Nonnull userId;
/** 回调周期内的音频丢帧率。 */
@property (nonatomic, assign) unsigned int packetLossRate;
/** 接收流的瞬时码率。 */
@property (nonatomic, assign) unsigned int recvBitrate;
/** 远端用户加入频道后音频卡顿的累计时长 （ms）。 */
@property (nonatomic, assign) unsigned int totalFrozenTimes;

@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 发言人音量信息。
 */
@interface DingRtcUserVolumeInfo : NSObject

/** 发言人的用户ID。 */
@property (nonatomic, copy) NSString * _Nonnull uid;
/** 发言状态，true：正在说话，false：没有说话。 */
@property (nonatomic, assign) BOOL speech_state;
/** 音量值，取值范围[0,255]。 */
@property (nonatomic, assign) int volume;

@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 入会鉴权信息。
 */
@interface DingRtcAuthInfo : NSObject <NSCopying>

/** 频道ID。 */
@property (nonatomic, copy) NSString * _Nonnull channelId;

/** 用户ID。 */
@property (nonatomic, copy) NSString * _Nonnull userId;
/** 应用ID。 */
@property (nonatomic, copy) NSString * _Nonnull appId;
/** 令牌。 */
@property (nonatomic, copy) NSString * _Nonnull token;
/** GSLB地址。 */
@property (nonatomic, copy) NSString * _Nullable gslbServer;

@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 显示视图配置。
*/
@interface DingRtcVideoCanvas : NSObject
/** 显示视图。 */
@property (nonatomic, strong) DingRtcOSView * _Nonnull view;
/** 渲染模式，默认值为{@link DingRtcRenderModeAuto}。 */
@property (nonatomic) DingRtcRenderMode renderMode;
/** 镜像模式，默认值为{@link DingRtcRenderMirrorModeAllDisabled}。 */
@property (nonatomic) DingRtcRenderMirrorMode mirrorMode;
/** 背景颜色，格式为RGB的Hex。 */
@property (nonatomic) int backgroundColor;
/** 旋转角度，默认值为 {@link DingRtcRotationMode_0}。 */
@property (nonatomic, assign) DingRtcRotationMode rotationMode;
/** 是否移除。*/
@property (nonatomic, assign) BOOL toBeRemoved;

@end

/**
 * @ingroup OC_DingRtcDef
 *
 * @since 3.0
 * @brief 水印配置。
*/
@interface DingRtcWatermarkConfig : NSObject
/** 预览水印是否可见。  */
@property (nonatomic, assign) BOOL visibleInPreview;
/** 竖屏水印坐标信息。  */
@property (nonatomic, assign) CGRect positionInPortraitMode;
/** 横屏水印坐标信息。  */
@property (nonatomic, assign) CGRect positionInLandscapeMode;
/** 水印透明度。  */
@property (nonatomic, assign) float alpha;
/** 归一化 。 */
@property (nonatomic, assign) BOOL normalized;

@end


/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 音频数据采样。
 */
@interface DingRtcAudioDataSample : NSObject

/** 音频数据。 */
@property (nonatomic, assign) void* _Nonnull data;
/** 采样点数（每个声道）。*/
@property (nonatomic, assign) int numOfSamples;
/** 每个采样点字节数，通常是16bit（即2字节）。*/
@property (nonatomic, assign) int bytesPerSample;
/** 声道数。 */
@property (nonatomic, assign) int numOfChannels;
/** 采样率。 */
@property (nonatomic, assign) int sampleRate;

@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 摄像头采集偏好。
 */
@interface DingRtcCameraCapturerConfiguration : NSObject

/** 采集偏好。 */
@property (nonatomic, assign) DingRtcCaptureOutputPreference preference;

#if TARGET_OS_IPHONE
/** 摄像头方向 (仅适用于 iOS 系统) 。*/
@property (nonatomic, assign) DingRtcCameraDirection cameraDirection;
#endif

@end

typedef NS_ENUM(NSUInteger, DingRtcLiveTranscodingDisplayType) {
    NOT_DISPLAY = 0,
    ALWAYS = 1,
    WHEN_NO_VIDEO = 2
};

/**
 * @brief 旁路直播图片。
 */
@interface DingRtcLiveTranscodingImage : NSObject
@property (nonatomic, copy) NSString *_Nullable url;
@property (nonatomic, assign) float alpha;
@property (nonatomic, assign) DingRtcLiveTranscodingDisplayType display;
@property (nonatomic, assign) int x;
@property (nonatomic, assign) int y;
@property (nonatomic, assign) int width;
@property (nonatomic, assign) int height;
@property (nonatomic, assign) int zOrder;
@end

typedef NS_ENUM(NSUInteger, DingRtcLiveTranscodingFontType) {
  NOTO_SERIF_CJKSC_REGULAR = 0,
  ALIBABA_PUHUITI_REGULAR = 1,
  ALIBABA_PUHUITI_BOLD = 2,
  ALIBABA_PUHUITI_Heavy = 3,
  ALIBABA_PUHUITI_LIGHT = 4,
  ALIBABA_PUHUITI_MEDIUM = 5
};

/**
 * @brief 旁路直播文字格式。
 */
@interface DingRtcLiveTranscodingText : NSObject
@property (nonatomic, copy) NSString *_Nullable text;
@property (nonatomic, assign) int x;
@property (nonatomic, assign) int y;
@property (nonatomic, assign) int zOrder;
@property (nonatomic, assign) DingRtcLiveTranscodingFontType fontType;
@property (nonatomic, assign) int fontSize;
@property (nonatomic, assign) int fontColor;//0xRRGGBB
@end

/**
 * @brief 旁路直播输入类型。
 */
typedef NS_ENUM(NSUInteger, DingRtcLiveTranscodingSegmentType) {
    /** 无人像分割。 */
    DingRtcLiveTranscodingNoBody = 0, 
    /** 人像分割。 */
    DingRtcLiveTranscodingBody = 1  
};

/**
 * @brief 旁路直播输入类型。
 */
typedef NS_ENUM(NSUInteger, DingRtcLiveTranscodingSourceType) {
    /** 相机流。 */
    DingRtcLiveTranscodingCamera = 0,
    /** 屏幕流。 */
    DingRtcLiveTranscodingShareScreen = 1
};

/**
 * @brief 旁路直播用户自定义窗格信息。
*/
@interface DingRtcLiveTranscodingPane : NSObject
@property (nonatomic, assign) int x;
@property (nonatomic, assign) int y;
@property (nonatomic, assign) int width;
@property (nonatomic, assign) int height;
@property (nonatomic, assign) int zOrder;
@property (nonatomic, copy) NSString *_Nonnull userId;
@property (nonatomic, assign) DingRtcLiveTranscodingSourceType sourceType;
@property (nonatomic, assign) DingRtcLiveTranscodingSegmentType segmentType;
@property (nonatomic, copy) NSArray<DingRtcLiveTranscodingImage *> *_Nullable images;
@property (nonatomic, copy) NSArray<DingRtcLiveTranscodingText *> *_Nullable texts;

@end

/** 旁路直播。 **/
typedef NS_ENUM(NSUInteger, DingRtcLiveTranscodingState) {
    DingRtcLiveTranscodingState_IDLE = 0,
    DingRtcLiveTranscodingState_CONNNECT = 1,
    DingRtcLiveTranscodingState_RUNNING = 2,
    DingRtcLiveTranscodingState_RECOVERING = 3,
    DingRtcLiveTranscodingState_FAILURE = 4,
    DingRtcLiveTranscodingState_END = 5
};

/**
 * @brief 旁路模式。
 */
typedef NS_ENUM(NSUInteger, DingRtcLiveTranscodingMixMode) {
    /** 单路模式。 */
    DingRtcLiveTranscodingSINGLE = 0,
    /** 混流模式。 */
    DingRtcLiveTranscodingMIX = 1
};

/**
 * @brief 旁路直播转推流模式。
 */
typedef NS_ENUM(NSUInteger, DingRtcLiveTranscodingStreamType) {
    /** 原始流。 */
    DingRtcLiveTranscodingOrigin = 0,
    /** 音频。 */
    DingRtcLiveTranscodingAudio = 1,
    /** 视频。 */
    DingRtcLiveTranscodingVideo = 2
};

/**
 * @brief 旁路直播媒体处理模式。
 */
typedef NS_ENUM(NSUInteger, DingRtcLiveTranscodingMediaProcessMode) {
    /** 通用模式。 */
    DingRtcLiveTranscodingNormal = 0,
    /** 虚拟背景模式。 */
    DingRtcLiveTranscodingVirtualBackground = 1
};

/**
 * @brief 旁路直播裁剪模式。
 */
typedef NS_ENUM(NSUInteger, DingRtcLiveTranscodingCropMode) {
    /** 剪裁。 */
    DingRtcLiveTranscodingCrop = 1,
    /** 填充。 */
    DingRtcLiveTranscodingFill = 2
};

/**
 * @brief 旁路直播时钟格式。
 */
@interface DingRtcLiveTranscodingClockWidget : NSObject
@property (nonatomic, assign) int x;
@property (nonatomic, assign) int y;
@property (nonatomic, assign) DingRtcLiveTranscodingFontType fontType;
@property (nonatomic, assign) int fontSize;
@property (nonatomic, assign) int fontColor;  // 0xRRGGBB
@property (nonatomic, assign) int zOrder;
@end

/**
 * @brief 旁路直播音频采样。
 */
typedef NS_ENUM(NSUInteger, DingRtcLiveTranscodingAudioSampleRate) {
    DingRtcLiveTranscoding_HZ_48000 = 0,
    DingRtcLiveTranscoding_HZ_44100 = 1,
    DingRtcLiveTranscoding_HZ_32000 = 2,
    DingRtcLiveTranscoding_HZ_16000 = 3,
    DingRtcLiveTranscoding_HZ_8000 = 4
};

/**
 * @brief 旁路直播自定义编码参数。
 */
@interface DingRtcLiveTranscodingEncodeParam : NSObject
@property (nonatomic, assign) int videoWidth; //(0-1920]
@property (nonatomic, assign) int videoHeight; //(0-1920]
@property (nonatomic, assign) int videoFramerate; //[1,60]
@property (nonatomic, assign) int videoBitrate; //[1kbps,10000kbps]
@property (nonatomic, assign) int videoGop; //[1,60]
@property (nonatomic, assign) DingRtcLiveTranscodingAudioSampleRate audioSamplerate;
@property (nonatomic, assign) int audioBitrate; //[8kbps,500kbps]
@property (nonatomic, assign) int audioChannels; //[1,2]
@end

/**
 * @brief 旁路直播计费规格。
 */
typedef NS_ENUM(NSUInteger, DingRtcLiveTranscodingTaskProfile) {
    DingRtcLiveTranscoding_Profile_1IN_1080P = 0,
    DingRtcLiveTranscoding_Profile_1IN_720P = 1,
    DingRtcLiveTranscoding_Profile_1IN_360P = 2,
    DingRtcLiveTranscoding_Profile_2IN_1080P = 3,
    DingRtcLiveTranscoding_Profile_2IN_720P = 4,
    DingRtcLiveTranscoding_Profile_2IN_360P = 5,
    DingRtcLiveTranscoding_Profile_4IN_1080P = 6,
    DingRtcLiveTranscoding_Profile_4IN_720P = 7,
    DingRtcLiveTranscoding_Profile_4IN_360P = 8,
    DingRtcLiveTranscoding_Profile_9IN_1080P = 9,
    DingRtcLiveTranscoding_Profile_9IN_720P = 10,
    DingRtcLiveTranscoding_Profile_9IN_360P = 11,
    DingRtcLiveTranscoding_Profile_12IN_1080P = 12,
    DingRtcLiveTranscoding_Profile_12IN_720P = 13,
    DingRtcLiveTranscoding_Profile_12IN_360P = 14,
    DingRtcLiveTranscoding_Profile_16IN_1080P = 15,
    DingRtcLiveTranscoding_Profile_16IN_720P = 16,
    DingRtcLiveTranscoding_Profile_16IN_360P = 17,
    DingRtcLiveTranscoding_Profile_Mixed = 9999 // only audio
};

/**
 * @brief 旁路模式单路参数。
 */
@interface DingRtcLiveTranscodingSingleParam : NSObject
/** 用户id。 */
@property (nonatomic, copy) NSString *_Nullable userId;
/** 推流模式。 */
@property (nonatomic, assign) DingRtcLiveTranscodingStreamType streamType;
/** 旁路直播输入类型。 */
@property (nonatomic, assign) DingRtcLiveTranscodingSourceType sourceType;
@end

/**
 * @brief 旁路模式混流参数。
 */
@interface DingRtcLiveTranscodingMixParam : NSObject
/** 旁路直播计费规格。 */
@property (nonatomic, assign) DingRtcLiveTranscodingTaskProfile taskProfile;
/** 旁路直播自定义编码参数。 */
@property (nonatomic, strong) DingRtcLiveTranscodingEncodeParam *_Nullable encodeParam;
/** 旁路直播用户列表。 */
@property (nonatomic, strong) NSArray<DingRtcLiveTranscodingPane *> *_Nullable users;
/** 背景色。 */
@property (nonatomic, assign) int backgroundColor; //0xRRGGBB
/** 背景图片列表。 */
@property (nonatomic, strong) NSArray<DingRtcLiveTranscodingImage *> *_Nullable backgrounds;
/** 水印列表。 */
@property (nonatomic, strong) NSArray<DingRtcLiveTranscodingImage *> *_Nullable watermarks;
/** 时钟组件列表。 */
@property (nonatomic, strong) NSArray<DingRtcLiveTranscodingClockWidget *> *_Nullable clockWidgets;
/** 剪裁模式。 */
@property (nonatomic, assign) DingRtcLiveTranscodingCropMode cropMode;
/** 媒体处理模式。 */
@property (nonatomic, assign) DingRtcLiveTranscodingMediaProcessMode mediaProcessMode;
@end

/**
 * @brief 旁路模式参数。
 */
@interface DingRtcLiveTranscodingParam : NSObject
/** 旁路模式。 */
@property (nonatomic, assign) DingRtcLiveTranscodingMixMode mixMode;
/** 路模式单路参数。 */
@property (nonatomic, strong) DingRtcLiveTranscodingSingleParam *_Nullable singleParam;
/** 旁路模式混流参数。 */
@property (nonatomic, strong) DingRtcLiveTranscodingMixParam *_Nullable mixParam;
@end

/**
 * @ingroup OC_DingRtcDef
 * 
 * @since 3.0
 * @brief 预定义视频分辨率。
 */
static const CGSize DingRtcVideoDimension120x120 = {120, 120};
static const CGSize DingRtcVideoDimension160x120 = {160, 120};
static const CGSize DingRtcVideoDimension180x180 = {180, 180};
static const CGSize DingRtcVideoDimension240x180 = {240, 180};
static const CGSize DingRtcVideoDimension320x180 = {320, 180};
static const CGSize DingRtcVideoDimension240x240 = {240, 240};
static const CGSize DingRtcVideoDimension320x240 = {320, 240};
static const CGSize DingRtcVideoDimension424x240 = {424, 240};
static const CGSize DingRtcVideoDimension360x360 = {360, 360};
static const CGSize DingRtcVideoDimension480x360 = {480, 360};
static const CGSize DingRtcVideoDimension640x360 = {640, 360};
static const CGSize DingRtcVideoDimension480x480 = {480, 480};
static const CGSize DingRtcVideoDimension640x480 = {640, 480};
static const CGSize DingRtcVideoDimension840x480 = {840, 480};
static const CGSize DingRtcVideoDimension960x540 = {960, 540};
static const CGSize DingRtcVideoDimension960x720 = {960, 720};
static const CGSize DingRtcVideoDimension1280x720 = {1280, 720};
static const CGSize DingRtcVideoDimension1920x1080 = {1920, 1080};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 相机流视频编码属性设置。
 */
@interface DingRtcVideoEncoderConfiguration : NSObject

/** 视频分辨率，默认值640x480，可自行设置分辨率，也可以选择上面预定义的分辨率。
*/
@property (nonatomic, assign) CGSize dimensions;

/** 视频帧率，默认值15, 最大值30。
*/
@property (nonatomic, assign) NSInteger frameRate;

/** 推流镜像，默认DingRtcVideoEncoderMirrorModeDisabled，详见 {@link DingRtcVideoEncoderMirrorMode}。
*/
@property (nonatomic, assign) DingRtcVideoEncoderMirrorMode mirrorMode;

/** 视频输出方向，默认DingRtcVideoEncoderOrientationModeAdaptive，详见 {@link DingRtcVideoEncoderOrientationMode}。
*/
@property (nonatomic, assign) DingRtcVideoEncoderOrientationMode orientationMode;

/** 推流旋转，默认DingRtcRotationMode_0，详见 {@link DingRtcRotationMode}。
*/
@property (nonatomic, assign) DingRtcRotationMode rotationMode;

@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 屏幕流编码属性设置。
 */
@interface DingRtcScreenShareEncoderConfiguration : NSObject

/** 视频分辨率，默认值0x0，最大值3840x2160。
 @note
 默认值表示推流分辨率等于屏幕采集的分辨率。
*/
@property (nonatomic, assign) CGSize dimensions;

/** 视频帧率，默认值5, 最大值30。
 */
@property (nonatomic, assign) NSInteger frameRate;

/** 推流旋转，默认DingRtcRotationMode_0，详见 {@link DingRtcRotationMode}。
*/
@property (nonatomic, assign) DingRtcRotationMode rotationMode;
@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 视频状态。
 */
typedef NS_ENUM(NSInteger, DingRtcVideoState){
    /** 视频关闭。 */
    DingRtcVideoClose   = 0, 
    /** 视频打开。 */
    DingRtcVideoOpen    = 1, 
};

/**
 * @brief 视频状态变化原因
 */
typedef NS_ENUM(NSInteger, DingRtcVideoReason){
    /** 由客户端触发。 */
    DingRtcVideoChangeByClient    = 0,
    /** 由服务器触发。 */
    DingRtcVideoChangeByServer    = 1,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief Qos反馈外置编码器请求帧类型
 */
typedef NS_ENUM(NSInteger, DingRtcVideoEncodedFrameType) {
    /** 默认 无。 */
    DingRtcVideoEncodedFrameNULL, 
    /** IDR帧。 */
    DingRtcVideoEncodedFrameIDR, 
    /** LTR帧。 */
    DingRtcVideoEncodedFrameLTR, 
    /** B帧。 */
    DingRtcVideoEncodedFrameB,  
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief Qos反馈外置编码器参数结构体。
 */
@interface DingRtcVideoExternalEncoderParameter : NSObject
/** 视频帧的宽。
 */
@property (nonatomic, assign) NSInteger width;
/** 视频帧的高。
 */
@property (nonatomic, assign) NSInteger height;
/** 编码帧率。
 */
@property (nonatomic, assign) NSInteger frame_rate;
/** 码率。
 */
@property (nonatomic, assign) NSInteger bitrate_bps;
@end

/**
 * @brief 视频美颜选项。
 */
@interface DingRtcBeautyFaceOptions : NSObject

/** 资源文件路径。 */ 
@property (nonatomic, strong) NSString * _Nonnull resourcePath;
/** 开启美肤。 */ 
@property (nonatomic, assign) BOOL enableSkinBuffing;
/** 磨皮系数。 */ 
@property (nonatomic, assign) float skinBuffingFactor;
/** 锐化系数。 */ 
@property (nonatomic, assign) float skinSharpenFactor;
/** 美白系数。 */ 
@property (nonatomic, assign) BOOL enableSkinWhitening;
/** 美白系数。 */ 
@property (nonatomic, assign) float skinWhitingFactor;
@end

/**
 * @brief 视频虚拟背景模式
 */
typedef NS_ENUM(NSInteger, DingRtcVirtualBackgroundMode) {
    /** 背景虚化。 */ 
    DingRtcVirtualBackgroundBlur,
    /** 背景替换。 */ 
    DingRtcVirtualBackgroundReplace,
};

/**
 * @brief 视频虚拟背景选项
 */
@interface DingRtcVirtualBackgroundOptions : NSObject

/** 虚拟背景模式。 */ 
@property (nonatomic, assign) DingRtcVirtualBackgroundMode mode;
/** 背景文件路径。 */ 
@property (nonatomic, strong) NSString * _Nonnull bgFilePath;
/** 资源文件路径。 */ 
@property (nonatomic, strong) NSString * _Nonnull resourcePath;
@end

/**
 * @brief 视频暗光增强选项。
 */
typedef NS_ENUM(NSInteger, DingRtcVideoEnhanceMode) {
    /** 视频暗光增强关。 */ 
    DingRtcVideoEnhanceOff,
    /** 视频暗光增强自动模式。 */ 
    DingRtcVideoEnhanceAuto,
};

/**
 * @brief 视频暗光增强选项。
 */
@interface DingRtcVideoEnhanceOptions : NSObject
/** 视频暗光增强模式。 */ 
@property (nonatomic, assign) DingRtcVideoEnhanceMode mode; 
@end

/**
 * @brief 视频去噪模式。
 */
typedef NS_ENUM(NSInteger, DingRtcVideoDenoiseMode) {
    /** 视频去噪关。 */ 
    DingRtcVideoDenoiseOff,  /*  */
    /** 视频去噪自动模式。 */ 
    DingRtcVideoDenoiseAuto, /*  */
};

/**
 * @brief 视频去噪选项。
 */
@interface DingRtcVideoDenoiseOptions : NSObject
/** 视频去噪模式。 */    
@property (nonatomic, assign) DingRtcVideoDenoiseMode mode;
@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 视频帧类型。
 */
typedef NS_ENUM(NSInteger, DingRtcVideoFrameType) {
  /** 裸数据类型。 */
  DingRtcVideoFrameRaw = 0,          /*  */
  /** CVPixelBufferRef 类型。 */
  DingRtcVideoFramePixelBuffer = 1,  /* CVPixelBufferRef 类型 */
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 视频数据格式。
 */
typedef NS_ENUM(NSInteger, DingRtcVideoPixelFormat) {
  DingRtcVideoI420,    /* I420 */
  DingRtcVideoNV12,    /* NV12 */
  DingRtcVideoBGRA,    /* BGRA */
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 视频帧数据结构。
 */
@interface DingRtcVideoFrame : NSObject

@property (nonatomic, assign) DingRtcVideoFrameType frameType;
@property (nonatomic, assign) DingRtcVideoPixelFormat pixelFormat;
/** 时间戳。 */
@property (nonatomic, assign) long long timestamp;
/** 视频帧的宽。 */
@property (nonatomic, assign) int width;
/** 视频帧的高。 */
@property (nonatomic, assign) int height;
/** 视频帧的旋转角度。 */
@property (nonatomic, assign) int rotation;
/** 视频帧的数量。 */
@property (nonatomic, assign) int count;
/** 视频帧数据。 */
@property (nonatomic, assign) void* _Nonnull data;
@property (nonatomic, strong) NSArray<NSNumber *> * _Nullable offset;
/** 视频帧行间距。 */
@property (nonatomic, strong) NSArray<NSNumber *> * _Nullable stride;
/** pixelBuffer数据。 */
@property (nonatomic, assign) CVPixelBufferRef _Nullable pixelBuffer;

@end

@interface DingRtcRecordOptions : NSObject
@property (nonatomic, assign) unsigned int width;
@property (nonatomic, assign) unsigned int height;
@property (nonatomic, assign) DingRtcRecordType recordMode;
@property (nonatomic, assign) int videoFps;
@property (nonatomic, assign) BOOL useHardwareAccelerate;
@end

/**
 * @brief 录制视频背景色 (仅适用于 Mac 系统)。
 */
typedef struct  {
    /** 红色分量。 */
    uint8_t r;
    /** 绿色分量。 */
    uint8_t g;
    /** 蓝色分量。 */
    uint8_t b;
} DingRtcRecordVideoBgColor;

/**
 * @brief 录制视频布局 (仅适用于 Mac 系统)。
 */
typedef NS_ENUM(NSUInteger, DingRtcRecordVideoLayoutMode) {
    /** 网格布局。 */
    DingRtcRecordVideoLayoutMode_GRID = 0,
    /** 演讲者布局。 */
    DingRtcRecordVideoLayoutMode_SPEAKER = 1,
    /** 只有一个画面布局。 */
    DingRtcRecordVideoLayoutMode_UNIQUE = 2,
    /** 自定义布局。 */
    DingRtcRecordVideoLayoutMode_CUSTOM = 3,
};

/**
 * @brief 录制视频布局Region (仅适用于 Mac 系统)。
 */
@interface DingRtcRecordVideoRegion : NSObject

@property (nonatomic, strong) NSString * _Nonnull userId;
@property (nonatomic, assign) DingRtcVideoSource sourceType;

@end

/**
 * @brief 录制视频布局 (仅适用于 Mac 系统)。
 */
@interface DingRtcRecordVideoLayout : NSObject
/** 视频布局模式。 */
@property (nonatomic, assign) DingRtcRecordVideoLayoutMode mode;
/** 录制视频背景色。 */
@property (nonatomic, assign) DingRtcRecordVideoBgColor backColor;
/** 录制视频用户布局数组。 */
@property (nonatomic, strong) NSMutableArray<DingRtcRecordVideoRegion *> * _Nullable shapes;

@end

#if TARGET_OS_MAC && !TARGET_OS_IPHONE

/**
 * @brief 录制视频模板 (仅适用于 Mac 系统)。
 */
@interface DingRtcRecordTemplate : NSObject
/** 录制类型。 */
@property (nonatomic, assign) DingRtcRecordType recordType;
/** 录制格式。 */
@property (nonatomic, assign) DingRtcRecordFormat recordFormat;
/** 音频采样率类型。 */
@property (nonatomic, assign) DingRtcAudioSampleRate sampleRate;
/** 音频录制质量。 */
@property (nonatomic, assign) DingRtcAudioQuality audioQuality;
/** 外部pcm采集。 */
@property (nonatomic, assign) bool enableRecordExternalCapturePCM;
/** 外部pcm渲染。 */
@property (nonatomic, assign) bool enableRecordExternalRenderPCM;
/** 录制视频分辨率。 */
@property (nonatomic, assign) DingRtcRecordVideoCanvasConfig canvas;
/** 帧率。 */
@property (nonatomic, assign) int fps;
/** 码率。 */
@property (nonatomic, assign) int bitrate;
/** 分段录制。 */
@property (nonatomic, assign) bool isFragment;

@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @remark mac
 * @brief 屏幕分享区域 (仅适用于 Mac 系统)。
 */
@interface DingRtcScreenShareRegion : NSObject

@property (nonatomic, assign) float originX;
@property (nonatomic, assign) float originY;
@property (nonatomic, assign) float width;
@property (nonatomic, assign) float height;

@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @remark mac
 * @brief 屏幕分享配置 (仅适用于 Mac 系统)。
 */
@interface DingRtcScreenShareConfig : NSObject

@property (nonatomic, assign) bool isShareByRegion;
@property (nonatomic, strong) DingRtcScreenShareRegion * _Nonnull shareRegion;

@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @remark mac
 * @brief 屏幕分享源信息 (仅适用于 Mac 系统)。
 */
@interface DingRtcScreenSourceInfo : NSObject

@property (nonatomic, copy) NSString * _Nonnull sourceId;
@property (nonatomic, copy) NSString * _Nonnull sourceName;

@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @remark mac
 * @brief 屏幕分享配置信息 (仅适用于 Mac 系统)。
 */
@interface DingRtcScreenShareInfo : NSObject

@property (nonatomic, assign) DingRtcScreenShareType sourceType;
@property (nonatomic, copy) NSString * _Nonnull sourceId;
@property (nonatomic, strong) DingRtcScreenShareConfig * _Nonnull config;

@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @remark mac
 * @brief 设备信息 (仅适用于 Mac 系统)。
 */
@interface DingRtcDeviceInfo : NSObject

/** 设备名称。 */
@property (nonatomic, copy) NSString * _Nullable deviceName;  
/** 设备ID。 */
@property (nonatomic, copy) NSString * _Nullable deviceID; 
/** 设备传输类型。 */
@property (nonatomic, assign) DingRtcDeviceTransportType deviceTransportType;

@end

#endif

#pragma mark - 视频观察器
/**
 * @defgroup OC_DingRtcCallbackVideoFrameObserver 视频数据回调及监听
 * @ingroup OC_DingRtcCallback
 * DingRtc 视频数据回调及监听
 * @{
 */
@protocol DingRtcVideoFrameDelegate <NSObject>
@optional
- (DingRtcVideoFormat)getVideoFormatPreference;
- (BOOL)onCaptureVideoFrame:(DingRtcVideoFrame * _Nonnull)frame;
@end
/**
 * @}
*/

#pragma mark - 音频观察器
/**
 * @defgroup OC_DingRtcCallbackAudioFrameObserver 音频数据回调及监听
 * @ingroup OC_DingRtcCallback
 * DingRtc 音频数据回调及监听
 * @{
 */
@protocol DingRtcAudioFrameDelegate <NSObject>

@optional
/**
 * @brief 本地订阅音频数据回调。
 * @details 远端所有用户混音后待播放的音频数据。
 * @param frame 音频数据。
 * @since 3.0
*/
- (void)onPlaybackAudioFrame:(DingRtcAudioDataSample * _Nonnull)frame;

/**
 * @brief 本地采集音频数据回调。
 * @param frame 音频数据。
 * @since 3.0
 */
- (void)onCapturedAudioFrame:(DingRtcAudioDataSample * _Nonnull)frame;

/**
 * @brief 经过3A处理后的数据回调。
 * @param frame 音频数据。
 * @since 3.0
*/
- (void)onProcessCapturedAudioFrame:(DingRtcAudioDataSample * _Nonnull)frame;

/**
 * @brief 本地推流音频数据回调。
 * @param frame 音频数据。
 * @since 3.0
 */
- (void)onPublishAudioFrame:(DingRtcAudioDataSample * _Nonnull)frame;

@end
/**
 * @}
*/
#pragma mark - 回调
/**
 * @defgroup OC_DingRtcCallbackEventListener 视频数据回调及监听
 * @ingroup OC_DingRtcCallback
 * DingRtc 回调及监听
 * @{
 */
@protocol DingRtcEngineDelegate <NSObject>
@optional

/**
 * @since 3.0
 * @brief 加入频道回调。
 * @details 当应用调用 {@link DingRtcEngine::joinChannel:name:onResultWithUserId:} 方法时，该回调表示成功/失败加入频道，并且返回频道加入的相关信息以及加入频道耗时。
 * @param result 加入频道结果
 *          - 0：成功。
 *          - 非0：失败。
 *   - {@link DingRtcErrJoinBadAppId} ：AppId不存在，请检查入会参数authinfo中appId是否正确。
 *   - {@link DingRtcErrJoinInvalidAppId} ：AppId已失效，请检查入会参数authinfo中appId是否正确。
 *   - {@link DingRtcErrJoinBadChannel} ：频道不存在，请检查入会参数authinfo中channelId是否正确。
 *   - {@link DingRtcErrJoinInvalidChannel} ：频道已失效，请检查入会参数authinfo中channelId是否正确。
 *   - {@link DingRtcErrJoinBadToken} ：token不存在，请检查入会参数authinfo生成算法是否与官网文档一致。
 *   - {@link DingRtcErrJoinTimeout} ：加入频道超时，请检查当前网络是否为弱网或限速。
 *   - {@link DingRtcErrJoinBadParam} ：参数错误，请检查入会参数authinfo生成是否存在非法字符，或与官网文档是否一致。
 *
 * @param channel 加入频道名。
 * @param userId   用户ID。
 * @param elapsed 加入频道耗时。
 * @note 此回调等同于joinChannel接口的block参数，二者择一处理即可。
 */
- (void)onJoinChannelResult:(int)result channel:(NSString *_Nonnull)channel userId:(NSString *_Nonnull)userId elapsed:(int)elapsed;

/**
 * @since 3.0
 * @brief 离开频道回调。
 * @details 调用 {@link leaveChannel} 方法后，该回调表示成功/失败离开频道，回调将会返回离会的result和该频道的基本信息。
 * @param result 离开频道结果，成功返回0，失败返回错误码。
 * @param stats 本次频道内会话的数据统计信息。
 * @note 调用 {@link DingRtcEngine::leaveChannel} 接口后返回，如果 {@link DingRtcEngine::leaveChannel} 后直接调用 {@link DingRtcEngine::destroy} 将不会收到此回调。
 */
- (void)onLeaveChannelResult:(int)result stats:(DingRtcStats *_Nonnull)stats;

/**
 * @since 3.0
 * @brief 音频推流变更回调。
 * @param oldState 之前的推流状态，详见{@link DingRtcPublishState}。
 * @param newState 当前的推流状态，详见{@link DingRtcPublishState}。
 * @param elapseSinceLastState 状态变更时间间隔（毫秒）。
 * @param channel 当前频道id。
 */
- (void)onAudioPublishStateChanged:(DingRtcPublishState)oldState newState:(DingRtcPublishState)newState elapseSinceLastState:(NSInteger)elapseSinceLastState channel:(NSString *_Nonnull)channel;

/**
 * @since 3.0
 * @brief 视频推流变更回调。
 * @param oldState 之前的推流状态，详见{@link DingRtcPublishState}。
 * @param newState 当前的推流状态，详见{@link DingRtcPublishState}。
 * @param elapseSinceLastState 状态变更时间间隔（毫秒）。
 * @param channel 当前频道id。
*/
- (void)onVideoPublishStateChanged:(DingRtcPublishState)oldState newState:(DingRtcPublishState)newState elapseSinceLastState:(NSInteger)elapseSinceLastState channel:(NSString *_Nonnull)channel;

/**
 * @since 3.0
 * @brief 屏幕分享推流变更回调。
 * @param oldState 之前的推流状态，详见{@link DingRtcPublishState}。
 * @param newState 当前的推流状态，详见{@link DingRtcPublishState}。
 * @param elapseSinceLastState 状态变更时间间隔（毫秒）。
 * @param channel 当前频道id。
*/
- (void)onScreenSharePublishStateChanged:(DingRtcPublishState)oldState newState:(DingRtcPublishState)newState elapseSinceLastState:(NSInteger)elapseSinceLastState channel:(NSString *_Nonnull)channel;

/**
 * @since 3.0
 * @brief 远端用户（通信模式）加入频道回调。
 * @details 该回调在以下场景会被触发。
 * - 通信模式：远端用户加入频道会触发该回调，如果当前用户在加入频道前已有其他用户在频道中，当前用户加入频道后也会收到已加入频道用户的回调。
 *
 * @param uid 用户ID 从App server分配的唯一标示符。
 * @param elapsed 用户加入频道时的耗时。
 */
- (void)onRemoteUserOnLineNotify:(NSString *_Nonnull)uid elapsed:(int)elapsed;

/**
 * @since 3.0
 * @brief 远端用户（通信模式）离开频道回调。
 * @details 该回调在以下场景会被触发。
 * - 通信模式：远端用户离开频道会触发该回调。
 * - 通信模式情况下，当长时间收不到远端用户数据，超时掉线时，会触发该回调。
 *
 * @param uid 用户ID 从App server分配的唯一标示符。
 * @param reason 用户离线的原因，详见 {@link DingRtcUserOfflineReason}。
 */
- (void)onRemoteUserOffLineNotify:(NSString *_Nonnull)uid offlineReason:(DingRtcUserOfflineReason)reason;

/**
 * @since 3.0
 * @brief 远端用户的音视频流发生变化回调。
 * @details 该回调在以下场景会被触发。
 * - 当远端用户从未推流变更为推流（包括音频和视频）。
 * - 当远端用户从已推流变更为未推流（包括音频和视频）。
 *
 * @param uid userId，从App server分配的唯一标示符。
 * @param audioTrack 音频流类型，详见 {@link DingRtcAudioTrack}。
 * @param videoTrack 视频流类型，详见 {@link DingRtcVideoTrack}。
 */
- (void)onRemoteTrackAvailableNotify:(NSString *_Nonnull)uid audioTrack:(DingRtcAudioTrack)audioTrack videoTrack:(DingRtcVideoTrack)videoTrack;

/**
 * @since 3.0
 * @brief 音频流订阅情况变更回调。
 * @param uid 用户ID，从App server分配的唯一标示符。
 * @param oldState 之前的订阅状态，详见 {@link DingRtcSubscribeState}。
 * @param newState 当前的订阅状态，详见 {@link DingRtcSubscribeState}。
 * @param elapseSinceLastState 两次状态变更时间间隔（毫秒）。
 * @param channel 当前频道id。
 */
- (void)onAudioSubscribeStateChanged:(NSString *_Nonnull)uid oldState:(DingRtcSubscribeState)oldState newState:(DingRtcSubscribeState)newState elapseSinceLastState:(NSInteger)elapseSinceLastState channel:(NSString *_Nonnull)channel;

/**
 * @since 3.0
 * @brief 相机流订阅情况变更回调。
 * @param uid 用户ID，从App server分配的唯一标示符。
 * @param oldState 之前的订阅状态，详见 {@link DingRtcSubscribeState}。
 * @param newState 当前的订阅状态，详见 {@link DingRtcSubscribeState}。
 * @param elapseSinceLastState 两次状态变更时间间隔（毫秒）。
 * @param channel 当前频道id。
 */
- (void)onVideoSubscribeStateChanged:(NSString *_Nonnull)uid oldState:(DingRtcSubscribeState)oldState newState:(DingRtcSubscribeState)newState elapseSinceLastState:(NSInteger)elapseSinceLastState channel:(NSString *_Nonnull)channel;

/**
 * @since 3.0
 * @brief 屏幕分享流订阅情况变更回调。
 * @param uid 用户ID，从App server分配的唯一标示符。
 * @param oldState 之前的订阅状态，详见 {@link DingRtcSubscribeState}。
 * @param newState 当前的订阅状态，详见 {@link DingRtcSubscribeState}。
 * @param elapseSinceLastState 两次状态变更时间间隔（毫秒）。
 * @param channel 当前频道id。
 */
- (void)onScreenShareSubscribeStateChanged:(NSString *_Nonnull)uid oldState:(DingRtcSubscribeState)oldState newState:(DingRtcSubscribeState)newState elapseSinceLastState:(NSInteger)elapseSinceLastState channel:(NSString *_Nonnull)channel;

/**
 * @since 3.0
 * @brief 相机流大小规格订阅情况变更回调。
 * @param uid 用户ID，从App server分配的唯一标示符。
 * @param oldStreamType 之前的订阅的规格类型，详见 {@link DingRtcVideoStreamType}。
 * @param newStreamType 当前的订阅的规格类型，详见 {@link DingRtcVideoStreamType}。
 * @param elapseSinceLastState 大小流类型变更时间间隔（毫秒）。
 * @param channel 当前频道id。
 */
- (void)onSubscribeStreamTypeChanged:(NSString *_Nonnull)uid oldStreamType:(DingRtcVideoStreamType)oldStreamType newStreamType:(DingRtcVideoStreamType)newStreamType elapseSinceLastState:(NSInteger)elapseSinceLastState channel:(NSString *_Nonnull)channel;

/**
 * @since 3.0
 * @brief 网络质量变化时发出的消息。
 * @param uid 网络质量发生变化的uid。
 * @param upQuality  上行网络质量。
 * @param downQuality  下行网络质量。
 * @note 当网络质量发生变化时触发，uid为""时代表self的网络质量变化。
 */
- (void)onNetworkQualityChanged:(NSString *_Nonnull)uid
               upNetworkQuality:(DingRtcNetworkQuality)upQuality
             downNetworkQuality:(DingRtcNetworkQuality)downQuality;

/**
 * @since 3.0
 * @brief 网络连接状态改变的回调。
 * @param status 当前状态值, 对应值参考枚举DingRtcConnectionStatus。
 * @param reason 引起状态变化的具体原因, 对应值参考枚举DingRtcConnectionStatusChangeReason。
*/
- (void)onConnectionStatusChanged:(DingRtcConnectionStatus)status reason:(DingRtcConnectionStatusChangeReason)reason;

/**
 * @since 3.0
 * @brief 被服务器踢出/会议结束频道的消息。
 * @param code 原因参考{@link DingRtcOnByeType}。
 */
- (void)onBye:(DingRtcOnByeType)code;

/**
 * @since 3.0
 * @brief 如果engine出现warning，通过这个回调通知app。
 * @param warn  Warning type。
 * @param message 警告描述。
 */
- (void)onOccurWarning:(int)warn message:(NSString *_Nonnull)message;

/**
 * @since 3.0
 * @brief 如果engine出现error，通过这个回调通知app。
 * @param error Error类型，参考{@link DingRtcErrorCode}。
 * @param message 错误描述。
 */
- (void)onOccurError:(int)error message:(NSString *_Nonnull)message;


/**
 * @since 3.0
 * @brief 音频首包发送回调。
 * @details 在首个音频数据包发送出去时触发此回调。
 * @param timeCost 发送耗时，从入会开始到音频首包发送出去的耗时。
 */
- (void)onFirstAudioPacketSentWithTimecost:(int)timeCost;

/**
 * @since 3.0
 * @brief 视频首包发送回调。
 * @param videoTrack 发送视频track，参考 {@link DingRtcVideoTrack}。
 * @param timeCost 耗时（毫秒）。
 * @note 该接口用于视频首包发送的回调。
*/
- (void)onFirstVideoPacketSentWithVideoTrack:(DingRtcVideoTrack)videoTrack
                                    timeCost:(int)timeCost;

/**
 * @since 3.0
 * @brief 远端用户音频首包接收回调。
 * @details 在接收到远端首个音频数据包时触发此回调。
 * @param uid 远端用户ID，从App server分配的唯一标识符。
 * @param timeCost 接收耗时，从入会开始到音频首包接收到的耗时。
 */
- (void)onFirstAudioPacketReceivedWithUid:(NSString *_Nonnull)uid
                                 timeCost:(int)timeCost;

/**
 * @since 3.0
 * @brief 远端用户视频首包接收回调。
 * @param uid User ID，从App server分配的唯一标示符。
 * @param videoTrack 接收视频track，参考 {@link DingRtcVideoTrack}。
 * @param timeCost 耗时（毫秒）。
 * @note 该接口用于视频首包接收的回调。
*/
- (void)onFirstVideoPacketReceivedWithUid:(NSString *_Nonnull)uid
                               videoTrack:(DingRtcVideoTrack)videoTrack
                                 timeCost:(int)timeCost;

/**
 * @since 3.0
 * @brief 远端用户视频首帧接收回调。
 * @param uid User ID，从App server分配的唯一标示符。
 * @param videoTrack 接收视频track，参考 {@link DingRtcVideoTrack}。
 * @param timeCost 耗时（毫秒）。
 * @note 该接口用于收到远端用户视频首帧的回调。
*/
- (void)onFirstVideoFrameReceivedWithUid:(NSString *_Nonnull)uid
                              videoTrack:(DingRtcVideoTrack)videoTrack
                                timeCost:(int)timeCost;

/**
 * @since 3.0
 * @brief 已解码远端音频首帧回调。
 * @param uid 远端用户ID，从App server分配的唯一标识符。
 * @param elapsed 从本地用户加入频道直至该回调触发的延迟, 单位为毫秒。
 */
- (void)onFirstRemoteAudioDecodedWithUid:(NSString *_Nonnull)uid elapsed:(int)elapsed;

/**
 * @since 3.0
 * @brief 远端用户的第一帧视频帧显示时触发这个消息。
 * @param uid User ID，从App server分配的唯一标示符。
 * @param videoTrack 屏幕流或者相机流，参考 {@link DingRtcVideoTrack}。
 * @param width 视频宽度。
 * @param height 视频高度。
 * @param elapsed 本地用户加入频道直至该回调触发的延迟总耗时（毫秒）。
 * @note 该接口用于远端用户的第一帧视频帧显示时的回调。
 */
- (void)onFirstRemoteVideoFrameDrawn:(NSString *_Nonnull)uid videoTrack:(DingRtcVideoTrack)videoTrack width:(int)width height:(int)height elapsed:(int)elapsed;

/**
 * @since 3.0
 * @brief 预览开始显示第一帧视频帧时触发这个消息。
 * @param width 本地预览视频宽度。
 * @param height 本地预览视频高度。
 * @param elapsed 从本地用户加入频道直至该回调触发的延迟总耗时（毫秒）。
 * @note 该接口用于预览开始显示第一帧视频帧时的回调。
 */
- (void)onFirstLocalVideoFrameDrawn:(int)width height:(int)height elapsed:(int)elapsed;

/**
 * @since 3.0
 * @brief 远端用户音量提示回调。
 * @param speakers 用户音量信息，表示回调用户音量信息数组，包含用户uid,语音状态以及音量，uid为"0"表示本地说话人。
 * @param speakerNumber 远端用户数量。
 */
- (void)onAudioVolumeIndication:(NSArray <DingRtcUserVolumeInfo *> *_Nullable)speakers speakerNumber:(int)speakerNumber;

/**
 * @since 3.0
 * @brief 远端用户静音/取消静音回调。
 * @param uid 远端用户ID。
 * @param isMute 
 * - 该用户是否静音
 * - true：静音。
 * - false：取消静音。
 */
- (void)onUserAudioMuted:(NSString *_Nonnull)uid audioMuted:(BOOL)isMute;

/**
 * @since 3.0
 * @brief 对端用户发送视频黑帧数据发送通知。
 * @param uid 执行muteVideo的用户。
 * @param isMute 
 *         -  YES：推流黑帧。
 *         -  NO：正常推流。
 * @param track 屏幕流或者相机流。
 * @note 该接口用于对端用户发送视频黑帧数据时的回调。
 */
- (void)onUserVideoMuted:(NSString *_Nonnull)uid videoMuted:(BOOL)isMute track:(DingRtcVideoTrack)track;

/**
 * @since 3.0
 * @brief 质量报告事件，相对onRtcLocalAudioStats, onRtcRemoteVideoStats更加简练，适合在UI上展示。
 * @param quality 采集到的各项指标数据。
 */
- (void)onRtcQualityReport:(DingRtcQualityReport *_Nonnull)quality;

/**
 * @since 3.0
 * @brief 实时数据回调（2s触发一次）。
 * @param stats stats。
 */
- (void)onRtcStats:(DingRtcStats *_Nonnull)stats;

/**
 * @since 3.0
 * @brief 本地视频统计信息（2s触发一次）。
 * @param localVideoStats 本地视频统计信息。
 * @note SDK每两秒触发一次此统计信息回调。
 */
- (void)onRtcLocalVideoStats:(DingRtcLocalVideoStats *_Nonnull)localVideoStats;

/**
 * @since 3.0
 * @brief 远端视频统计信息（2s触发一次）。
 * @param remoteVideoStats 远端视频统计信息。
 */
- (void)onRtcRemoteVideoStats:(DingRtcRemoteVideoStats *_Nonnull)remoteVideoStats;

/**
 * @since 3.0
 * @brief 本地音频统计信息（2s触发一次）。
 * @param localAudioStats 本地视频统计信息。
 * @note SDK每两秒触发一次此统计信息回调。
 */
- (void)onRtcLocalAudioStats:(DingRtcLocalAudioStats *_Nonnull)localAudioStats;

/**
 * @since 3.0
 * @brief 远端音频统计信息（2s触发一次）。
 * @param remoteAudioStats 远端音频统计信息。
 */
- (void)onRtcRemoteAudioStats:(DingRtcRemoteAudioStats *_Nonnull)remoteAudioStats;

#if TARGET_OS_IPHONE
/**
 * @since 3.0
 * @remark iOS
 * @brief 音频路由发生变化回调（仅适用于 iOS 系统）。
 * @details 所谓“音频路由”，是指声音是从手机的扬声器还是从听筒中播放出来，音频路由变化也就是声音的播放位置发生了变化。
 * - 当音频路由为听筒时，声音比较小，只有将耳朵凑近才能听清楚，隐私性较好，适合用于接听电话。
 * - 当音频路由为扬声器时，声音比较大，不用将手机贴脸也能听清，因此可以实现“免提”的功能。
 *
 * @param routing 当前使用的音频路由，详情参考{@link DingRtcAudioRouteType}
 */
- (void)onAudioRouteChanged:(DingRtcAudioRouteType)routing;
#endif

#if TARGET_OS_MAC && !TARGET_OS_IPHONE

/**
 * @since 3.0
 * @remark mac
 * @brief 音频设备状态变更 (仅适用于 Mac 系统)。
 * @param deviceInfo  外接设备信息。
 * @param deviceType  外接设备类型。
 * @param deviceState 外接设备状态。
 */
- (void)onAudioDeviceStateChanged:(DingRtcDeviceInfo *_Nonnull)deviceInfo deviceType:(DingRtcExternalDeviceType)deviceType deviceState:(DingRtcExternalDeviceState)deviceState;

/**
 * @since 3.0
 * @remark mac
 * @brief 音频采集设备测试回调 (仅适用于 Mac 系统)。
 * @param level 音频采集设备音量值。
 */
- (void)onAudioDeviceRecordLevel:(int)level;

/**
 * @since 3.0
 * @remark mac
 * @brief 音频播放设备测试回调 (仅适用于 Mac 系统)。
 * @param level 音频采集设备音量值。
 */
- (void)onAudioDevicePlayoutLevel:(int)level;

#endif

/**
 * @since 3.0
 * @brief API 方法已执行回调。
 * @param error 当该方法调用失败时 SDK 返回的错误码。如果该方法调用成功，SDK 将返回 0。失败，非0。
 * @param api SDK 执行的 API。
 * @param result SDK 调用 API 的调用结果。
 */
- (void)onApiCalledExecuted:(int)error api:(NSString *_Nullable)api result:(NSString *_Nullable)result;

/**
 * @since 3.0
 * @brief 视频截图回调。
 * @param uid 用户uid，为nil或者为""代表本地用户。
 * @param videoTrack 截图视频track，参考 {@link DingRtcVideoTrack}。
 * @param filePath 截图的本地保存路径。
 * @param width 截图宽度。
 * @param height 截图高度。
 * @param image 截图数据。
 * @param success 截图结果。
 * @note 成功时image返回截图数据，失败返回nil。
 */
- (void)onSnapshotComplete:(NSString*_Nullable)uid videoTrack:(DingRtcVideoTrack)videoTrack filePath:(NSString* _Nullable)filePath width:(int)width height:(int)height success:(BOOL)success;


@end
/** @}  */

#pragma mark - 接口主类

@interface DingRtcEngine : NSObject <DingRtcEngineDelegate>

#pragma mark - "Engine的生命周期，工作参数以及频道操作"
/**
 * @name Engine的生命周期，工作参数以及频道操作
 * @{
 */

/**
 * @ingroup OC_DingRtcEngineBase
 *
 * @since 3.0
 * @brief 获取一个DingRtcEngine实例。
 * @param delegate 用于接收DingRtc相关时间回调的代理。
 * @param extras 用于接收来自客户灰度下发的参数，通过JSON配置SDK的特别功能，可以是空字符串。详情请参见官网[有关extras功能说明](https://help.aliyun.com/document_detail/200048.htm?spm=a2c4g.11186623.2.111.4b89466bJosuro#task-2025192) 。
 * @details 该方法为同步调用，以单例的形式提供对象创建。
 * @note 注意 同一时间只会存在一个主实例。
 * @note 创建子实例，请参考使用{@link createChannel}。
 */
+ (instancetype _Nonnull )sharedInstance:(id<DingRtcEngineDelegate>_Nullable)delegate extras:(NSString *_Nullable)extras;

/**
 * @ingroup OC_DingRtcEngineBase
 *
 * @since 3.0
 * @brief 销毁 {@link DingRtcEngine}。
 * @details 调用该方法将会释放所有内部使用的资源,当完成音视频通信之后都建议调用该方法释放实例.调用该方法后，你将不能再使用 {@link DingRtcEngine} 的其他方法和任何回调，如需要再次使用需要重新调用 {@link DingRtcEngine::sharedInstance} 方法创建一个新的实例。
 * @note 该方法为同步调用,需要等待内部资源释放之后才能执行其他方法,为避免主线程阻塞，建议开发者放在子线程调用该方法，但需要注意的是如需在销毁后再次创建 {@link DingRtcEngine} 实例，请开发者务必保证 destroy 方法执行结束后再创建实例。
 * @note 为避免死锁，不建议在任何SDK的回调中调用本方法。
 */
+ (void)destroy;

/**
 * @ingroup OC_DingRtcEngineBase
 *
 * @since 3.0
 * @brief 查询SDK当前版本号。
 * @return 当前的SDK版本号，格式为字符串，例："3.0.0.0"。
 * @note 在任何时刻都可以调用。
 */
+ (NSString *_Nonnull)getSDKVersion;

/**
 * @ingroup OC_DingRtcEngineBase
 *
 * @since 3.0
 * @brief 设置SDK日志文件保存路径。
 * @param logDirPath 日志文件保存目录的绝对路径，该参数是一个目录路径，而不是文件的路径。
 * @return
 * - 0，成功；
 * - 非0，失败。其中有两种错误码，释义如下：
 *   - {@link DingRtcErrLogRootPathError}：路径已存在或因权限问题不可访问。
 *   - {@link DingRtcErrLogAlreadyInit}：log对象已经初始化过。
 *
 * @note 调用此接口，请在调用其他所有SDK接口前调用本接口，避免日志出现丢失。同时，App必须保证指定的目录已存在并且可以写入。
 * @note 如果，目标路径不可写入或创建日志文件失败，则会使用默认日志路径。本次设置将无效。
 */
+ (int)setLogDirPath:(NSString *_Nullable)logDirPath;

/**
 * @ingroup OC_DingRtcEngineBase
 *
 * @since 3.0
 * @brief 设置日志等级。
 * @param logLevel Log级别，详见{@link DingRtcLogLevel}枚举类型。
 * @return
 * - 0：成功；
 * - 非0：失败。比如没有先调用setLogDirPath。
 *
 * @note 设置 SDK 的输出日志等级。默认值为 {@link DingRtcLogLevel::DingRtcLogLevelInfo}。
 * @note 调用此接口，请在仅次于SetLogDirPath接口调用之后再调用本接口，避免因日志等级切换造成的打印丢失。
 */
+ (int)setLogLevel:(DingRtcLogLevel)logLevel;

/**
 * @ingroup OC_DingRtcEngineOthers
 *
 * @since 3.0
 * @brief 根据错误码，获取错误码描述。
 * @param errCode 错误码，详见{@link DingRtcErrorCode}。
 * @return 错误码描述字符串。
 * @note 在任何时刻都可以调用。
 */
+ (NSString *_Nullable)getErrorDescription:(NSInteger)errCode;

/**
 * @ingroup OC_DingRtcEngineOthers
 *
 * @since 3.0
 * @brief 是否分发回调到主线程。
 * @param enabled 
 *          - YES：回调分发至主线程队列。
 *          - NO：回调不分发至主线程队列。
 * @note 回调默认分发至主线程队列（iOS Default:YES，Mac Default:NO）。
 * @return 0表示Success；非0表示Failure。
 */
- (int)enableDelegateMainQueue:(BOOL)enabled;

/**
 * @ingroup OC_DingRtcEngineOthers
 *
 * @since 3.0
 * @brief 指定回调线程队列。
 * @param queue 回调queue。
 * @note enableDelegateMainQueue为YES时，此接口设置无效，回调线程为主线程。
 * @note enableDelegateMainQueue为NO时，可通过此接口指定回调线程，若不设置，则使用SDK的默认子线程。
 * @return 0表示Success；非0表示Failure。
 */
- (int)setDelegateQueue:(NSOperationQueue *_Nullable)queue;

/**
 * @ingroup OC_DingRtcEngineChannel
 *
 * @since 3.0
 * @brief 加入频道。
 * @details 该方法让用户加入频道，一个频道内可以加入多个用户进行互相通话。
 * @param authInfo   认证信息，从App Server获取。
 * @param userName   任意用于显示的用户名称。不是用户ID。
 * @param onResult   当joinChannel执行结束后调用这个回调。
 * @return
 * - 0：成功；
 * - 非0：失败。
 * @note
 *    - 正常情况一个Appid对应一个App，只有同一个AppId的应用可进行互相童话,不同AppId和channelID不同都不能互通。
 *    - 当入会成功后，会触发远端用户收到 {@link onRemoteUserOnLineNotify:elapsed:} 回调。
 *    - 如果已在会中想重新入会，需要先调用 {@link leaveChannel} 离会，且确保 {@link onLeaveChannelResult:stats:} 回调收到后，再次调用进行入会。
 */
- (int)joinChannel:(DingRtcAuthInfo *_Nonnull)authInfo name:(NSString *_Nullable)userName onResultWithUserId:(void(^_Nullable)(NSInteger errCode, NSString * _Nonnull channel, NSString * _Nonnull userId, NSInteger elapsed))onResult;

/**
 * @ingroup OC_DingRtcEngineChannel
 *
 * @since 3.0
 * @brief  离开频道。
 * @details 调用该方法代表离开频道，即挂断或者停止通话。
 * @details 该方法为异步方法，调用后收到 {@link onLeaveChannelResult:stats:} 代表退出频道完成，同时会触发远端用户收到 {@link onRemoteUserOffLineNotify:offlineReason:}   回调。
 * @details 如果未加入频道入会，{@link leaveChannel} 不会有任何影响。加入频道后，必须要调用该方法离开频道，否则无法加入其他频道。
 * @return
 * - 0：成功；
 * - 非0：失败。
 *   - 失败原因错误码见回调函数：{@link  onLeaveChannelResult:stats:}。
 * @note
 *  - 如果调用了 {@link leaveChannel} 后立刻调用 {@link DingRtcEngine::destroy} 方法，SDK将会无法触发 {@link  onLeaveChannelResult:stats:} 回调。
 *
 */
- (int)leaveChannel;

/** @} */

#pragma mark - "设置和操作本地的媒体"
/**
 * @name 设置和操作本地的媒体
 * @{
 */

/**
 * @ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @brief 设置本地预览显示视图。
 * @param viewConfig 本地视频显示属性, 详细见{@link DingRtcVideoCanvas}。
 * @param track  只支持{@link DingRtcVideoTrackCamera}和{@link DingRtcVideoTrackScreen}。
 * @return
* - 0 ：成功；
 * - < 0 ：失败。
 * @note 支持joinChannel之前和之后切换窗口。如果canvas或者{@link DingRtcVideoCanvas::view}为nil，则停止显示。
 *       如果需要重新设置{@link DingRtcVideoCanvas::renderMode}，请保持canvas中其他属性不变，仅修改
 *       {@link DingRtcVideoCanvas::renderMode}。
 *       如果需要重新设置{@link DingRtcVideoCanvas::mirrorMode}，请保持canvas中其他属性不变，仅修改
 *       {@link DingRtcVideoCanvas::mirrorMode}。
 */
- (int)setLocalViewConfig:(DingRtcVideoCanvas *_Nullable)viewConfig forTrack:(DingRtcVideoTrack)track;

/**
 * @ingroup OC_DingRtcEnginePreview
 *
 * @since 3.0
 * @brief 开启本地视频预览。
 * @return
 * - 0 ：成功；
 * - < 0 ：失败。
 * @note 如果没有调用{@link #setLocalViewConfig:forTrack:}设置显示的view，则无法预览。
 * 可以在{@link #joinChannel:name:onResult:}之前就开启预览会自动打开摄像头。
 */
- (int)startPreview;

/**
 * @ingroup OC_DingRtcEnginePreview
 *
 * @since 3.0
 * @brief 关闭本地视频预览。
 * @return
 * - 0 ：成功；
 * - < 0 ：失败。
 * @note {@link #leaveChannel}会自动停止本地预览；如果没有在推相机流，则会自动关闭摄像头。
 */
- (int)stopPreview;

/**
 * @brief 检查camera是否打开。
 * @return
 * - YES：表示摄像头已打开；
 * - NO：表示摄像头没有打开。
 * @note 该接口用于检查当前camera的状态。
 */
- (BOOL)isCameraOn;

/**
 * @ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @brief 设置camera采集偏好。
 * @param config 
 *      - 偏好设置  
 *      - preference：{@link DingRtcCaptureOutputPreference::DingRtcCaptureOutputPreferencePreview} 高清预览，采集优先保证视频预览质量。
 *                    {@link DingRtcCaptureOutputPreference::DingRtcCaptureOutputPreferencePerformance} 采集选择最接近推流的分辨率，优先保证设备性能。
 *                    {@link DingRtcCaptureOutputPreference::DingRtcCaptureOutputPreferenceAuto} 自动调整采集分辨率。
 *      - cameraDirection：设置采集方向，前置 或 后置。
 * @return 0为成功；非0失败。
 * @note 必须在打开摄像头之前设置，如{@link DingRtcEngine::startPreview}，{@link DingRtcEngine::joinChannel:name:onResult:} / {@link DingRtcEngine::joinChannel:name:onResultWithUserId:}之前设置。
*/
- (int)setCameraCapturerConfiguration:(DingRtcCameraCapturerConfiguration* _Nonnull )config;

/**
 * @ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @brief 禁用或启用本地视频采集。
 * @param enabled
 * - YES ：启用本地视频采集。
 * - NO ：禁用本地视频采集。
 * @return
 * - 0 ：成功；
 * - < 0 ：失败。
 * @note 默认为开启状态。
 */
- (int)enableLocalVideo:(BOOL)enabled;

/**
 * @ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @brief 是否将停止本地视频数据发送。
 * @param mute  
 *         - YES表示视频数据发送黑帧。
 *         -  NO表示恢复正常。
 * @param track  只支持{@link DingRtcVideoTrackCamera}。
 * @return
 * - 0：表示Success；
 * - 非0：表示Failure。
 * @note 发送黑色的视频帧。本地预览也呈现黑色。采集，编码，发送模块仍然工作，
 *       只是视频内容是黑色帧。
*/
- (int)muteLocalCamera:(BOOL)mute forTrack:(DingRtcVideoTrack)track;

/**
 * @ingroup OC_DingRtcEnginePub
 *
 * @since 3.0
 * @brief 是否推送本地视频流。
 * @param enabled 
 * - 是否开启/关闭本地视频流推流。
 * - YES：开启视频流推送。
 * - NO：关闭视频流推送。
 * @return
 * - 0：设置成功；
 * - <0：设置失败。
 *   - DingRtcErrInner：SDK内部状态错误，需检查是否创建SDK实例成功。
 *
 * @note
 * - SDK默认设置为推送视频流，加入频道前调用此接口设置推送视频流，将在加入频道成功时生效，加入频道成功后可以随时调用此接口开启/关闭视频流推送。
 * - 本地视频流内容默认为摄像头采集视频，如需自定义输入视频推送内容，可参考接口{@link DingRtcEngine::setExternalVideoSource:track:}使用外部视频输入替换摄像头采集。
 * - 本地视频推流结果发生变化时，SDK会触发{@link DingRtcEngineDelegate::onVideoPublishStateChanged:newState:elapseSinceLastState:channel:}回调通知视频推流最新状态。
*/
- (int)publishLocalVideoStream:(BOOL)enabled;

/**
 * @ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @brief 设置相机流视频编码属性。
 * @details 该方法用于设置相机流视频编码属性对应的视频参数，如分辨率、帧率、码率、视频方向等 所有设置的参数都有相应的范围限制，如果设置的参数不在有效范围内，SDK会自动调节。
 * @param config 预定义的编码属性，详见{@link DingRtcVideoEncoderConfiguration}。
 * @note
 * - 该方法在入会前和入会后都可以调用，如果每次入会只需要设置一次相机流视频编码属性，建议在入会前调用。
*/
- (void)setVideoEncoderConfiguration:(DingRtcVideoEncoderConfiguration* _Nonnull)config;

/**
 * @ingroup OC_DingRtcEngineScreenShare
 *
 * @since 3.0
 * @brief 设置屏幕流视频编码属性。
 * @details 该方法用于设置屏幕流视频编码属性对应的视频参数，如分辨率、帧率、码率、视频方向等 所有设置的参数都有相应的范围限制，如果设置的参数不在有效范围内，SDK会自动调节。
 * @param config 预定义的屏幕共享编码属性，详见{@link DingRtcScreenShareEncoderConfiguration}。
 * @note
 * - 该方法在入会前和入会后都可以调用，如果每次入会只需要设置一次屏幕流视频编码属性，建议在入会前调用。
*/
- (void)setScreenShareEncoderConfiguration:(DingRtcScreenShareEncoderConfiguration* _Nonnull)config;

/**
 * @brief 开启或关闭视频美颜。
 * @private
 * @param enable 是否开启。
 * @param options 美颜参数。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)enableBeautyFace:(BOOL)enable withOption:(DingRtcBeautyFaceOptions* _Nonnull)options;

/**
 * @brief 设置视频暗光增强功能。
 * @private
 * @param options 视频暗光增强参数。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)setVideoEnhance:(DingRtcVideoEnhanceOptions* _Nonnull)options;

/**
 * @since 3.0
 * @brief 设置视频去噪场景功能。
 * @private
 * @param options 视频去噪参数。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)setVideoDenoise:(DingRtcVideoDenoiseOptions* _Nonnull)options;

/**
 * @ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @brief 设置视频帧观察器。
 * @param delegate DingRtcVideoFrameDelegate对象。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)setVideoFrameDelegate:(id<DingRtcVideoFrameDelegate> _Nullable)delegate;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @brief 停止/恢复本地音频数据发送。
 * @param mute  
 *          - YES表示静音本地音频，发送静音帧。
 *          - NO表示取消静音。
 * @param mode  静音模式，静音模式分三种，详见{@link DingRtcMuteLocalAudioMode}, 默认模式为静音本地麦克风采集音频和所有外部输入音频。
 * @return
 * - 0：成功；
 * - 非0：失败。
 * @note 静音是指音频流发送静音帧,采集和编码模块仍然在工作。
 */
- (int)muteLocalMic:(BOOL)mute mode:(DingRtcMuteLocalAudioMode)mode;

/**
 * @ingroup OC_DingRtcEnginePub
 *
 * @since 3.0
 * @brief 是否推送音频流。
 * @param enabled 
 * - 是否开启/关闭本地音频流推流。
 * - YES：开启音频流推送。
 * - NO：关闭音频流推送。
 * @return
 * - 0：设置成功；
 * - <0：设置失败。
 *  - DingRtcErrInner：SDK内部状态错误，需检查是否创建SDK实例成功。
 *
 * @note
 * - SDK默认设置推送音频流，加入频道前调用此接口设置推送音频流，将在加入频道成功时生效，加入频道成功后可以随时调用此接口开启/关闭音频流推送。
 * - 本地音频流内容默认为麦克风采集音频，如需自定义输入音频推送内容，可参考接口{@link DingRtcEngine::setExternalAudioSource:withSampleRate:channelsPerFrame:}使用外部音频输入替换麦克风采集。
 * - 本地音频推流结果发生变化时，SDK会触发{@link DingRtcEngineDelegate:onAudioPublishStateChanged:newState:elapseSinceLastState:channel:}回调通知音频推流最新状态。
 */
- (int)publishLocalAudioStream:(BOOL)enabled;

/** @} */

#pragma mark - "操作来自远端的媒体"
/**
 * @name 操作来自远端的媒体
 * @{
 */

/**
 * @ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @brief 设置远端视频显示视图。
 * @param canvas 本地视频显示属性, 详细见{@link DingRtcVideoCanvas}。
 * @param uid User ID，从App server分配的唯一标示符。
 * @param track  只支持{@link DingRtcVideoTrack#DingRtcVideoTrackCamera}和{@link DingRtcVideoTrack#DingRtcVideoTrackScreen}。
 * @return
 * - 0 ：成功；
 * - < 0 ：失败。
 * @note 支持joinChannel之前和之后切换窗口。如果canvas或者{@link DingRtcVideoCanvas#view}为nil，则停止显示。
 *       如果需要重新设置{@link DingRtcVideoCanvas#renderMode}，请保持canvas中其他属性不变，仅修改
 *       {@link DingRtcVideoCanvas#renderMode}。
 *       如果需要重新设置{@link DingRtcVideoCanvas#mirrorMode}，请保持canvas中其他属性不变，仅修改
 *       {@link DingRtcVideoCanvas#mirrorMode}。
 */
- (int)setRemoteViewConfig:(DingRtcVideoCanvas *_Nullable)canvas uid:(NSString *_Nonnull)uid forTrack:(DingRtcVideoTrack)track;


/** @} */

#pragma mark - "用户管理"
/**
 * @name 用户管理
 * @{
 */

/**
 * @ingroup OC_DingRtcEnginePub
 *
 * @since 3.0
 * @brief 查询是否设置推送视频流。
 * @return
 * - YES：已设置推送；
 * - NO：未设置推送。
 * @note
 *  - SDK默认设置推送视频流，如果需要取消推送视频流，可以通过{@link  DingRtcEngine::publishLocalVideoStream:}方法设置为不推送。
*/
- (BOOL)isLocalVideoStreamPublished;

/**
 * @ingroup OC_DingRtcEnginePub
 *
 * @since 3.0
 * @brief 查询是否设置推送音频流。
 * @return
 * - YES：已设置推送；
 * - NO：未设置推送。
 * @note
 *  - SDK默认设置推送音频流，如果需要取消推送音频流，可以通过{@link DingRtcEngine::publishLocalAudioStream:}方法设置为不推送。
*/
- (BOOL)isLocalAudioStreamPublished;

#if TARGET_OS_IPHONE
/**
 * @ingroup OC_DingRtcEngineScreenShare
 *
 * @since 3.0
 * @remark ios
 * @brief 启动屏幕分享  (仅适用于 iOS 系统)。
 * @param appGroup 插件和宿主app约定使用相同的appGroup。
 * @param mode 屏幕共享模式。
 * @return
 * - 0：成功，
 * - 其他：失败。
 */
- (int)startScreenShare:(NSString * _Nullable)appGroup
                   mode:(DingRtcScreenShareMode)mode;
#endif

/**
 * @ingroup OC_DingRtcEngineScreenShare
 *
 * @since 3.0
 * @brief 停止屏幕分享。
 * @return
 * - 0：成功；
 * - 其他：失败。
 */
- (int)stopScreenShare;

/**
 * @ingroup OC_DingRtcEngineScreenShare
 *
 * @since 3.0
 * @brief 查询是否设置推送屏幕分享。
 * @return
 * - YES：已设置推送；
 * - NO：未设置推送。
 * @note
 *  - SDK默认设置不推送屏幕分享，如果需要推送屏幕共享，可以通过{@link DingRtcEngine::startScreenShare}方法开启屏幕共享推送。
*/
- (BOOL)isScreenSharePublished;

#if TARGET_OS_MAC && !TARGET_OS_IPHONE
/**
 * @ingroup OC_DingRtcEngineScreenShare
 *
 * @since 3.0
 * @remark mac
 * @brief 根据桌面Id进行屏幕分享 (仅适用于 Mac 系统)。
 * @param desktopId 桌面Id(可通过GetScreenShareSourceInfo接口获取)。
 * @param config 屏幕分享配置，详见 {@link DingRtcScreenShareConfig}。
 * @return
 *  - 0：设置成功，
 *  - <0：设置失败，返回错误码。
 */
- (int)startScreenShareWithDesktopId:(NSString * _Nonnull)desktopId config:(DingRtcScreenShareConfig * _Nonnull)config;

/**
 * @ingroup OC_DingRtcEngineScreenShare
 *
 * @since 3.0
 * @remark mac
 * @brief 根据窗口id进行屏幕分享 (仅适用于 Mac 系统)。
 * @param windowId 窗口id(可通过GetScreenShareSourceInfo接口获取)。
 * @param config 屏幕分享配置，详见 {@link DingRtcScreenShareConfig}。
 * @return
 *  - 0：设置成功，
 *  - <0：设置失败，返回错误码。
 */
- (int)startScreenShareWithWindowId:(NSString * _Nonnull)windowId config:(DingRtcScreenShareConfig * _Nonnull)config;

/**
 * @ingroup OC_DingRtcEngineScreenShare
 *
 * @since 3.0
 * @remark mac
 * @brief 获取屏幕分享源信息 (仅适用于 Mac 系统)。
 * @param type 屏幕分享类型，详见 {@link DingRtcScreenShareType}。
 * @return 返回屏幕共享源列表。
 */
- (NSArray<DingRtcScreenSourceInfo *> * _Nullable)getScreenShareSourceInfoWithType:(DingRtcScreenShareType)type;
#endif

/**
 * @ingroup OC_DingRtcEngineSub
 *
 * @since 3.0
 * @brief 设置订阅视频流分辨率规格。
 * @param uid  用户ID，从App server分配的唯一标示符。
 * @param streamType 视频流的分辨率规格，详见 {@link DingRtcVideoStreamType}。
 * @return
 * - 0：设置成功；
 * - 非0：设置失败。
 */
- (int)setRemoteVideoStreamType:(NSString *_Nonnull)uid type:(DingRtcVideoStreamType)streamType;

/**
 * @ingroup OC_DingRtcEngineSub
 *
 * @since 3.0
 * @brief 设置默认订阅的视频流分辨率规格；如果不设置，则默认订阅最大的分辨率规格。
 * @param streamType 视频流的分辨率规格，详见 {@link DingRtcVideoStreamType}。
 * @return
 * - 0：设置成功；
 * - <0：设置失败。
 */
- (int)setRemoteDefaultVideoStreamType:(DingRtcVideoStreamType)streamType;

/**
 * @ingroup OC_DingRtcEngineSub
 *
 * @since 3.0
 * @brief 停止/恢复订阅所有远端音频流，包含后入会成员的订阅。
 * @param sub 
 * - 是否订阅所有远端音频流。
 * - YES：订阅所有用户的音频流，后入会成员会自动订阅其音频。
 * - NO：停止订阅所有用户的音频流，后入会成员也不会自动订阅其音频。
 * @return
 * - 0：设置成功；
 * - <0：设置失败。
 *
 * @note
 * - 入会前入会后均可以调用。
 */
- (int)subscribeAllRemoteAudioStreams:(BOOL)sub;

/**
 * @ingroup OC_DingRtcEngineSub
 *
 * @since 3.0
 * @brief 停止/恢复订阅所有远端视频流，包含后入会成员的订阅。
 * @param sub 
 * - 是否订阅所有远端视频流。
 * - YES：订阅所有用户的视频流，后入会成员会自动订阅其视频。
 * - NO：停止订阅所有用户的视频流，后入会成员也不会自动订阅其视频。
 * @return
 * - 0：设置成功；
 * - <0：设置失败。
 *
 * @note
 * - 入会前入会后均可以调用。
 * - 如果设置了 subscribeAllRemoteVideoStreams:NO，会取消订阅已订阅的远端视频流，同时后面再入会的人不会进行自动订阅；但是依然可以通过 {@link subscribeRemoteVideoStream:track:sub:} 来主动订阅某个人的远端视频流。
 */
- (int)subscribeAllRemoteVideoStreams:(BOOL)sub;

/**
 * @ingroup OC_DingRtcEngineSub
 * 
 * @since 3.0
 * @brief 停止/恢复订阅远端用户的视频流。
 * @param uid 用户ID，从App server分配的唯一标示符。
 * @param track 视频流类型，详见 {@link DingRtcVideoTrack}。
 * @param sub 
 * - 是否订阅远端用户的视频流。
 * - YES：订阅指定用户的视频流。
 * - NO：停止订阅指定用户的视频流。
 * @return
 * - 0：设置成功；
 * - <0：设置失败。
 *
 * @note
* - 如果设置了 subscribeAllRemoteVideoStreams:NO，会取消订阅已订阅的远端视频流，同时后面再入会的人不会进行自动订阅；但是依然可以通过 {@link subscribeRemoteVideoStream:track:sub:} 来主动订阅某个人的远端视频流。
 */
- (int)subscribeRemoteVideoStream:(NSString *_Nonnull)uid track:(DingRtcVideoTrack)track sub:(BOOL)sub;

/**
 * @ingroup OC_DingRtcEngineUserInfo
 *
 * @since 3.0
 * @brief 获取远端在线用户列表。
 * @return 当前频道内所有在线用户userid列表。
 * @note 若当前频道ChannelProfile设置为{@link DingRtcInteractivelive}模式，只有设置角色为{@link DingRtcClientRoleInteractive}的用户为在线用户，本方法不返回角色为{@link DingRtcClientRoleLive}的用户。
*/
- (NSArray<NSString *> *_Nullable)getOnlineRemoteUsers;

/**
 * @ingroup OC_DingRtcEngineUserInfo
 *
 * @since 3.0
 * @brief 查询用户是否在线。
 * @param uid 用户ID，从App server分配的唯一标示符。不可为NULL。
 * @return
 *  - YES，用户在线；
 *  - NO，用户不在线。
 * @note 根据特定的UserID，查询该用户是否在线。
 */
- (BOOL)isUserOnline:(NSString *_Nonnull)uid;

/**
 * @ingroup OC_DingRtcEngineOthers
 *
 * @since 3.0
 * @brief 获取当前网络链接状态。
 * @return 当前链接状态, 参考{@link DingRtcConnectionStatus}。
 * @note 网络链接状态发生变化时，SDK会通过{@link onConnectionStatusChanged}回调通知。
 * @note 入会前调用此接口，均返回状态DingRtcConnectionStatusDisconnected。
*/
- (DingRtcConnectionStatus)getCurrentConnectionStatus;

/** @} */

#pragma mark - "设备管理"
/**
 * @name 设备管理
 * @{
 */

#if TARGET_OS_IPHONE
/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark ios
 * @brief 设置音频输出为听筒还是扬声器 (仅适用于 iOS 系统)。
 * @param enable YES:扬声器模式(默认扬声器)；NO:听筒模式。
 * @return
 * - 0：成功；
 * - <0：失败。
 */
- (int)enableSpeakerphone:(BOOL)enable;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark ios
 * @brief 获取当前音频输出为听筒还是扬声器 (仅适用于 iOS 系统)。
 * @return
 * - YES：扬声器模式；
 * - NO：听筒模式。
 */
- (BOOL)isSpeakerphoneEnabled;
#endif

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark ios
 * @brief 设置音频profile。
 * @param audio_profile 音频采集/编码模式参数, 详见{@link DingRtcAudioProfile}。
 * @param audio_scene 音频场景模式参数,详见{@link DingRtcAudioScenario}。
 * @return
 * - 0：成功；
 * - 其他：失败。
 * @note 只支持joinChannel前设置有效。
 */
- (int)setAudioProfile:(DingRtcAudioProfile)audio_profile audio_scene:(DingRtcAudioScenario)audio_scene;

#if TARGET_OS_IPHONE

/**
 * @ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @remark ios
 * @brief 切换前后摄像头 (仅适用于 iOS 系统)。
 * @return
 * - 0：表示Success；
 * - 非0：表示Failure。
 */
- (int)switchCamera;

/**
 * @brief 设置摄像头缩放。
 * @param zoom   缩放级别，范围从 1 到 camera支持的最大 zoom值。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
 - (int)setCameraZoom:(float)zoom;

/**
 * @brief 设置摄像头闪光灯开关。
 * @param flash  是否打开闪光灯。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)setCameraFlash:(BOOL)flash;

/**
 * @brief 摄像头是否支持手动聚焦。
 * @return 
 * - YES：支持；
 * - NO：不支持。
 */
- (BOOL)isCameraFocusPointSupported;

/**
 * @brief 设置摄像头手动聚焦点。
 * @param point   聚焦点坐标，（相对坐标， 取值范围0~1）。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)setCameraFocusPoint:(CGPoint)point;

/**
 * @brief 摄像头是否支持设置曝光点。
 * @return 
 * - YES：支持；
 * - NO：不支持。
 */
- (BOOL)isCameraExposurePointSupported;

/**
 * @brief 设置摄像头曝光点。
 * @param point  曝光点，（相对坐标， 取值范围0~1）。
 * @return
 * - 0：成功；
 * - 非0：失败。
 */
- (int)setCameraExposurePoint:(CGPoint)point;

/**
 * @ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @remark ios
 * @brief 设置设备方向 (仅适用于 iOS 系统)。
 * @param orientation 设备方向。
 */
- (void)setDeviceOrientationMode:(DingRtcOrientationMode)orientation; 

#endif

#if TARGET_OS_MAC && !TARGET_OS_IPHONE
/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 获取系统中的录音设备列表 (仅适用于 Mac 系统)。
 */
- (NSArray<DingRtcDeviceInfo *> *_Nullable)getAudioCaptures;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 获取使用的录音设备名称 (仅适用于 Mac 系统)。
 */
- (NSString *_Nullable)getCurrentAudioCapture;

/**
 * @brief 获取使用的录音设备ID (仅适用于 Mac 系统)。
 */
- (NSString *_Nullable)getCurrentAudioCaptureID;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 选择录音设备(Name) (仅适用于 Mac 系统)。
 * @param capture  音频采集设备名称。
 */
- (void)setCurrentAudioCapture:(NSString *_Nonnull)capture;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 选择录音设备(ID) (仅适用于 Mac 系统)。
 * @param captureID  音频采集设备Id。
 */
- (void)setCurrentAudioCaptureWithID:(NSString *_Nonnull)captureID;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 获取系统中的扬声器列表 (仅适用于 Mac 系统)。
 */
- (NSArray<DingRtcDeviceInfo *> *_Nullable)getAudioRenderers;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 获取当前使用的扬声器名称 (仅适用于 Mac 系统)。
 */
- (NSString *_Nullable)getCurrentAudioRenderer;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 获取当前使用的扬声器ID (仅适用于 Mac 系统)。
 */
- (NSString *_Nullable)getCurrentAudioRendererID;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 选择扬声器(Name) (仅适用于 Mac 系统)。
 * @param renderer  音频播放设备名称。
 */
- (void)setCurrentAudioRenderer:(NSString *_Nonnull)renderer;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 选择扬声器(ID) (仅适用于 Mac 系统)。
 * @param rendererID  音频播放设备id。
 */
- (void)setCurrentAudioRendererWithID:(NSString *_Nonnull)rendererID;

/**
 * @ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @remark mac
 * @brief 获取摄像头列表 (仅适用于 Mac 系统)。
 * @return 找到的系统中摄像头名称列表。
 */
- (NSArray<DingRtcDeviceInfo *> *_Nullable)getCameraList;

/**
 * @ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @remark mac
 * @brief 获取摄像头列表 (仅适用于 Mac 系统)。
 * @return 回调方式找到的系统中摄像头名称列表。
 */
- (int)getCameraList:(void (^)(NSArray<DingRtcDeviceInfo *> *_Nullable))onResult;

/**
 * @ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @remark mac
 * @brief 获取当前使用的摄像头名称 (仅适用于 Mac 系统)。
 */
- (NSString *_Nullable)getCurrentCamera;

/**
 * @ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @remark mac
 * @brief 获取当前使用的摄像头ID (仅适用于 Mac 系统)。
 */
- (NSString *_Nullable)getCurrentCameraID;

/**
 * @ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @remark mac
 * @brief 选择摄像头(Name) (仅适用于 Mac 系统)。
 * @param camera   摄像头名称。
 */
- (void)setCurrentCamera:(NSString *_Nonnull)camera;

/**
 * @ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @remark mac
 * @brief 选择摄像头(ID) (仅适用于 Mac 系统)。
 * @param cameraID   摄像头ID。
 */
- (void)setCurrentCameraWithID:(NSString *_Nonnull)cameraID;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 设置音频采集设备音量 (仅适用于 Mac 系统)。
 * @param volume 音量范围 [0, 100]。
 * @return 0表示Success；非0表示Failure。
 */
- (int)setRecordingDeviceVolume:(NSInteger)volume;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 获取音频采集设备音量 (仅适用于 Mac 系统)。
 * @return 音频采集设备音量，音量范围 [0, 100]，返回-1表示失败。
 */
- (int)getRecordingDeviceVolume;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 设置音频播放设备音量 (仅适用于 Mac 系统)。
 * @param volume 音量范围 [0, 100]
 * @return 0表示Success；非0表示Failure。
 */
- (int)setPlayoutDeviceVolume:(NSInteger)volume;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 获取音频播放设备音量 (仅适用于 Mac 系统)。
 * @return 音频播放设备音量，音量范围 [0, 100]，返回-1表示失败。
 */
- (int)getPlayoutDeviceVolume;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 静音音频采集设备 (仅适用于 Mac 系统)。
 * @param mute TRUE：设备设置为静音，FALSE：设备设置为不静音。
 * @return 0：方法调用成功；<0：方法调用失败。
 */
- (int)setRecordingDeviceMute:(BOOL)mute;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 获取音频采集设备静音状态 (仅适用于 Mac 系统)。
 * @return TRUE：已静音状态；FALSE： 非静音状态。
 */
- (BOOL)getRecordingDeviceMute;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 静音音频播放设备 (仅适用于 Mac 系统)。
 * @param mute TRUE：设备设置为静音，FALSE：设备设置为不静音。
 * @return 0：方法调用成功；<0：方法调用失败。
*/
- (int)setPlaybackDeviceMute:(BOOL)mute;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 获取音频播放设备静音状态 (仅适用于 Mac 系统)。
 * @return TRUE：已静音状态；FALSE：非静音状态。
*/
- (BOOL)getPlaybackDeviceMute;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 开始测试音频采集设备 (仅适用于 Mac 系统)。
 * @return 0表示Success；非0表示Failure。
 * @note 请在joinChannel之前调用。
 */
- (int)startTestAudioRecord;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 停止测试音频采集设备 (仅适用于 Mac 系统)。
 * @return 0表示Success；非0表示Failure。
 * @note 请在joinChannel之前调用。
 */
- (int)stopTestAudioRecord;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 开始测试音频播放设备 (仅适用于 Mac 系统)。
 * @param deviceName 音频播放设备Name。
 * @param filePath 音频文件路径。
 * @param loopCycles 循环次数，可以设置-1(无限循环)或者>0的正整数次，其他值无效。
 * @return 0表示Success；非0表示Failure。
 * @note 请在joinChannel之前调用。
 */
- (int)startTestAudioPlayoutWithName:(NSString *_Nonnull)deviceName filePath:(NSString *_Nonnull)filePath loopCycles:(NSInteger)loopCycles;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @remark mac
 * @brief 停止测试音频播放设备 (仅适用于 Mac 系统)。
 * @return 0表示Success；非0表示Failure。
 * @note 请在joinChannel之前调用。
 */
- (int)stopTestAudioPlayout;

#endif

/** @} */

#pragma mark - "其他"
/**
 * @name 其他
 * @{
 */

/**
 * @ingroup OC_DingRtcEngineChannel
 *
 * @since 3.0
 * @brief 获取状态：当前是否在频道中。
 * @return
 *  - YES，表示当前已经加入了频道。
 *  - NO, 表示当前不在频道中。
 * @note 该方法可在加入频道前或加入频道后调用。
 */
- (BOOL)isInCall;

/**
 * @ingroup OC_DingRtcEngineUserInfo
 *
 * @since 3.0
 * @brief 查询远端用户状态。
 * @param uid 用户ID。从App server分配的唯一标示符。
 * @return dict App提供的容器，用于存放用户数据，dict类型为字典类型，通过{Key,Value}进行访问。
 *  - dict[userID] != @""  成功。
 *  - dict[userID] == @""  失败。
 * @note 通过 uid 查询，指定远端用户的状态信息，在参数dict中，以字典{key,value}的形式，返回用户的全部状态信息。
 * @note Key信息：
 *  - userID：远端用户的ID。
 *  - isOnline：远端用户是否在线。
 *  - displayName：远端用户的名称。
 *  - hasAudio：远端用户是否推送了音频流。
 *  - hasCamera：远端用户是否推送了相机流。
 *  - hasScreenSharing：远端用户是否推送了屏幕流。
 *  - requestAudio：本端是否订阅了此远端用户的音频流。
 *  - requestCamera：本端是否订阅了此远端用户的相机流。此状态，系在成功订阅该用户摄像头流前的状态。
 *  - requestScreenSharing：本端是否订阅了此远端用户的屏幕流。此状态，系在成功订阅该用户屏幕共享前的状态。
 *  - preferCameraType：如果网络带宽允许，优先订阅的相机流的大小规格，选择范围：FHD, HD, SD, LD。
 *  - subScribedAudio：本端是否拉到了此远端用户的音频流。
 *  - subScribedCameraType：本端拉到了此远端用户的相机流的大小规格，可能的值：FHD, HD, SD, LD。 
 *  - subScribedScreenSharing：本端是否拉到了此远端用户的屏幕流。
 *  - hasCameraView：本端是否设置了此远端用户相机流的view。
 *  - hasScreenView：本端是否设置了此远端用户屏幕流的view。
 *  - isCameraMirror：本端是否设置了此远端用户的相机流为镜像。
 *  - isScreenMirror：本端是否设置了此远端用户的屏幕流为镜像。
 */
- (NSDictionary *_Nullable)getUserInfo:(NSString *_Nonnull)uid;

/** @} */

#pragma mark - "音视频裸数据输入输出"
/**
 * @name 音视频裸数据输入输出
 * @{
 */

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @brief 设置音量回调频率和平滑系数。
 * @param interval 时间间隔，单位毫秒，最小值不得小于10ms, 建议设置300-500ms, <= 0表示不启用音量提示和说话人提示功能。
 * @param smooth 平滑系数，数值越大平滑程度越高，反之越低，实时性越好，建议设置3，范围[0, 9]。
 * @param reportVad 
 * - 说话人检测开关
 * - 1：开启。
 * - 0：关闭。
 * @return
 * - 0：成功；
 * - <0：失败。
 * @note 设置之后，音频音量通过{@link DingRtcEngineDelegate::onAudioVolumeIndication:}回调。
 */
- (int)enableAudioVolumeIndication:(NSInteger)interval smooth:(NSInteger)smooth reportVad:(NSInteger)reportVad;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @brief 设置是否启用外部音频输入源。
 * @param enable 
 *          - YES：开启。
 *          - NO：关闭。
 * @param sampleRate 采样率 16k, 48k...
 * @param channelsPerFrame 声道数 1：单声道; 2：双声道。
 * @return
 * - >=0：成功；
 * - <0：失败。
 */
- (int)setExternalAudioSource:(BOOL)enable withSampleRate:(NSUInteger)sampleRate channelsPerFrame:(NSUInteger)channelsPerFrame;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @brief 输入外部音频数据推流到远端。
 * @param data 音频数据采样，详见 {@link DingRtcAudioDataSample}。
 * @return
 * - <0：表示失败；
 * - 0：表示成功。
 * - 返回值为{@link DingRtcErrorCode::DingRtcErrAudioBufferFull}时，需要在间隔投递数据时间长度后再次重试投递。
 * @note 为了保证语音连续性，SDK内部会有buffer缓存送进来的音频数据，以每次送10ms数据为例，如果返回{@link DingRtcErrorCode::DingRtcErrAudioBufferFull}时，说明内部缓冲区已满，建议等待20ms后再重新送当前数据帧。
 */
- (int)pushExternalAudioFrame:(DingRtcAudioDataSample *_Nonnull)data;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @brief 注册音频数据回调。
 * @private
 * @param observer 音频数据回调，delegate = nil 则停止回调数据。
 * @note 需要调用enableAudioFrameObserver开启具体DingRtcAudioSource的回调。
 * @return
 *  - 0：成功；
 *  - 非0：失败。
 */
- (int)registerAudioFrameObserver:(id<DingRtcAudioFrameDelegate> _Nullable)observer;

/**
 * @since 3.0
 * @brief 订阅音频数据。
 * @private
 * @param enable 是否允许数据回调。
 * @param audioSource 音频裸数据源类型，详见 {@link DingRtcAudioSource}。
 * @return 0：sucess。
 */
- (int)enableAudioFrameObserver:(BOOL)enable audioSource:(DingRtcAudioSource)audioSource;

#pragma mark - "文件录制"
/**
 * @name 文件录制
 * @{
 */

#if TARGET_OS_MAC && !TARGET_OS_IPHONE

/**
 * @brief 开始录制 (仅适用于 Mac 系统)
 * @details SDK提供了开始录制的功能
 * @param recordOptions 录制视频内容及布局, 详细定义见{@link DingRtcRecordOptions}
 * @param filePath 录制文件的本地保存路径
 * @param fileName 录制文件名
 * @return
 * - YES：成功
 * - NO：失败
 */
- (BOOL)startRecord:(DingRtcRecordOptions*_Nonnull)recordOptions filePath:(NSString*_Nullable)filePath fileName:(NSString*_Nullable)fileName;

/**
 * @brief 更新录制内容信息 (仅适用于 Mac 系统)。
 * @details SDK提供了更新录制内容信息的功能,用于实时更新录制模板内容设置
 * @param layout 录制视频内容及布局, 详细定义见{@link DingRtcRecordVideoLayout}
 * @return
 * - YES：成功，
 * - NO：失败。
 * @note updateRecordLayout更新录制内容信息应该在录制过程中调用
 */
- (BOOL)updateRecordLayout:(DingRtcRecordVideoLayout*_Nonnull)layout;

/**
 * @brief 暂停录制 (仅适用于 Mac 系统)。
 * @details SDK提供了暂停录制的功能。
 * @return
 * - YES：成功，
 * - NO：失败。
 */
- (BOOL)pauseRecord;

/**
 * @brief 重新开启录制 (仅适用于 Mac 系统)。
 * @details SDK提供了重新开始录制的功能。
 * @return
 * - YES：成功，
 * - NO：失败。
 */
- (BOOL)resumeRecord;

/**
 * @brief 停止录制 (仅适用于 Mac 系统)
 * @details SDK提供了停止录制的功能
 */
- (void)stopRecord;

#endif

/** @} */

#pragma mark - "external video source"
/**
 * @name external video source
 * @{
 */
/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @brief 设置外部视频源。
*/
- (int)setExternalVideoSource:(BOOL)enable track:(DingRtcVideoTrack)track;

/**
 * @ingroup OC_DingRtcEngineAudio
 *
 * @since 3.0
 * @brief 推送外部视频数据。
*/
- (int)pushExternalVideoFrame:(DingRtcVideoFrame*_Nonnull)frame track:(DingRtcVideoTrack)track;

/** @} */

#pragma mark - "视频画面截图"
/**
 * @name 视频画面截图
 * @{
 */

/**
 *@ingroup OC_DingRtcEngineVideo
 *
 * @since 3.0
 * @brief 对指定用户的视频进行截图，生成一张 JPG 格式的图片，并保存至指定的路径。
 * @param userId 用户ID，userId 为nil或者为""代表本地用户。
 * @param type 视频流类型，只支持 {@link DingRtcVideoTrack::DingRtcVideoTrackCamera} 和 {@link DingRtcVideoTrack::DingRtcVideoTrackScreen}。
 * @param filePath 截图的本地保存路径，需精确到文件名及格式，请确保目录存在且可写。如 xx/yy/zz.jpg。
 * @return
 * - 0：仅代表调用该接口成功，是否获取到截图画面，详细见 {@link DingRtcEngineDelegate::onSnapshotComplete:videoTrack:image:success:}；
 * - < 0：失败。
 * @note 视频截图功能为异步接口, 通过监听 {@link DingRtcEngineDelegate::onSnapshotComplete:videoTrack:image:success:} 回调获取当前截图画面。
 */
- (int)snapshotVideo:(NSString*_Nullable)userId type:(DingRtcVideoTrack)type filePath:(NSString* _Nonnull)filePath;

/** @} */

@end
